import 'dart:async';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:flutter/material.dart';
import 'package:rxdart/subjects.dart';
import 'package:socket_io_client/socket_io_client.dart' as io_soc;
import 'package:socket_io_client/socket_io_client.dart';

import '../../model/group_list_model.dart';

io_soc.Socket socket = io_soc.io(Global.baseUrl,
    OptionBuilder().setTransports(['websocket']).disableAutoConnect().build());
StreamSocket chatListSoket = StreamSocket();
StreamSocket receivedMessageSocket = StreamSocket();

class StreamSocket {
  StreamController<dynamic> socketResponse = BehaviorSubject();
  void Function(dynamic) get addResponse => socketResponse.sink.add;
  Stream<dynamic> get getResponse => socketResponse.stream;
}


void getGroupList() {
  var params = {
    "secretKey": AppStorages.secKey,
    "userId": AppStorages.myUserId
  };
  socket.emit('get-group-list', params);
  socket.on('get-group-list-res', (data) {
    print(data);
    chatListSoket.addResponse(data);
    var groupListModel =
    groupListModelFromJson(data);
    if(groupListModel.length==1){
      socket.emit('online-user', { "secretKey": AppStorages.secKey,
        "userId": AppStorages.myUserId});
    }
  });
}

void getConversation(
    {groupId, userId, startAt, Function(dynamic success)? onSuccess}) {
  var convoParam = {
    "groupId": groupId,
    "userId": userId,
    "startAt": startAt,
    "secretKey": AppStorages.secKey,
  };
  socket.emit('get-chat', convoParam);
  socket.on('get-chat-res', (data) {

    if (data != null) {
      onSuccess!(data);
    }
  });
}

void sendMessage(param) async {
  socket.emit('send-message', param);
}

void offline() async {
  var params = {
    "secretKey": AppStorages.secKey,
    "userId": AppStorages.myUserId
  };
  socket.emit('offline', params);
}

void onlineUser() async {
  var params = {
    "secretKey": AppStorages.secKey,
    "userId": AppStorages.myUserId
  };
  socket.emit('online', params);
}

void receivedMessages({Function(dynamic success)? success}) {

  socket.on('receive-message', (data) {
print(data);
    if (data != null) {
      success!(data);
    }
  });
}

void typingSocket({groupId, userId, secKey, name, isTyping}) {
  var typeParam = {
    'groupId': groupId,
    'userId': userId,
    "secretKey": secKey,
    "name": name,
    "isTyping": isTyping
  };
  socket.emit('typing', typeParam);
}

void typingResSocket({Function(dynamic onSuccess)? success}) {
  socket.on("typing-res", (data) {
    if (data != null) {
      success!(data);
    }
  });
}

void userActiveStatus({Function(dynamic onSuccess)? success}) {
  socket.on("online-status", (data) {
    if (data != null) {
      success!(data);
    }
  });
}

void clearChat({groupId, userId, Function? onSuccess}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
    "secretKey": AppStorages.secKey
  };
  socket.emit("clear-chat", params);
  socket.on("clear-chat-res", (data) {
    if (data['isSuccess']) {
      if (onSuccess != null) {
        onSuccess();
      }
    }
  });
}

void leaveConvo({groupId, userId}) async {
  var leaveConvoParam = {
    "groupId": groupId,
    "userId": userId,
    "secretKey": AppStorages.secKey
  };
  socket.emit('leave-group', leaveConvoParam);
}

void getGroup({groupId, Function(dynamic success)? onSuccess}) {
  var groupDetailParam = {
    "groupId": groupId,
    "userId": AppStorages.myUserId,
    "secretKey": AppStorages.secKey
  };
  socket.emit('get-group', groupDetailParam);
  socket.on('get-group-res', (data) {
    if (data != null) {
      onSuccess!(data);
    }
  });
}

void updateGroupInfo(param, {context}) {

  socket.emit("update-group", param);
  socket.on("update-group-res", (data) {
    Navigator.pushReplacement(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) =>  ChatsListView()));
  });
}

void exitGroup({groupId, userId, context}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
    "secretKey":AppStorages.secKey
  };
  socket.emit("exit-group", params);
  socket.on("exit-group-res", (data) {
    if (data['isSuccess']) {
      // Get.off(() => const ChatsListView());
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) =>  ChatsListView()));
    }
  });
}

void deleteGroup({groupId, userId, context}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
    "secretKey":AppStorages.secKey
  };
  socket.emit("delete-group", params);
  socket.on("delete-group-res", (data) {
    if (data['isSuccess']) {
      // Get.off(() => const ChatsListView());
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) =>  ChatsListView()));
    }
  });
}
void deleteChat({groupId, userId, context}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
    "secretKey":AppStorages.secKey
  };
  socket.emit("delete-chat", params);
  socket.on("delete-chat-res", (data) {
    if (data['isSuccess']) {
      // Get.off(() => const ChatsListView());
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) =>  ChatsListView()));
    }
  });
}
