import 'dart:convert';
import 'package:aplus_chat/core/services/stream_socket.dart';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/model/set_user_role_model.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:aplus_chat/view/message_view.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:socket_io_client/socket_io_client.dart';

import '../view/widgets/themes/cometchat_theme.dart';
import '../view/widgets/themes/palette.dart';

class APlusChatMain {
  final String? secretKey;
  final String? userId;

  ///Constructor for creating CometChat object
  /// [appId] refers to your CometChat App ID.
  /// [region] The region where the app was created.
  APlusChatMain(
    this.secretKey, {
    this.userId,
  });

  /// ... INIT APlUS CHAT  (Add in you main page of project)... ///
  static Future aPlusChatInit({required String secKey}) async {
    await AppStorages.initializeApp();
    socket.connect();
    socket.onDisconnect((_) {
      socket.io
        ..disconnect()
        ..connect();
    });

     AppStorages.myUserId = AppStorages.getChatUserId();

    AppStorages.secKey = secKey;
    AppStorages.myUserName.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserName);
    AppStorages.myEmail.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserEmail);
    AppStorages.myFirstName.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserFirstName);
    AppStorages.myLastName.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserLastName);
    AppStorages.myProfilePicture.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserProfilePic);
    AppStorages.myEmail.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserEmail);

    AppStorages.myRoles = AppStorages.getChatUserRoles() ??
        SetUserRole(
          sendMessage: 1,
          deleteChat: 0,
          deleteMessage: 0,
          updateProfile: 0,
          editMessage: 0,
          createGroup: 0,
          createOneToOneChat: 0,
        );
  }

  /// ... OPEN CHAT SCREEN ... ///
  static openAPlusChat({required BuildContext? context,required bool? isDark,required Color? themeColor}) {
    Navigator.push(
      context!,
      MaterialPageRoute(builder: (context) =>  ChatsListView(theme: CometChatTheme(palette: Palette(mode: isDark!?PaletteThemeModes.dark:PaletteThemeModes.light,accent: PaletteModel(light: themeColor!, dark: themeColor))),
    )));
  }

  /// ... SET USER DATA ... ///
  static setAPlusUserData({
    required String userId,
    required String fcmToken,
    required String userName,
    required String secretKey,
    required String email,
    firstName,
    lastName,
    @required mobile,
    profilePicture,
    SetUserRole? setUserRole,
    @required Function(dynamic value)? onSuccess,
    @required Function(dynamic value)? onError,
    @required Function(dynamic value)? onConnectionFailed,
  }) {
    AppStorages.setChatUserRoles(setUserRole ??
        SetUserRole(
          deleteMessage: 0,
          deleteChat: 0,
          sendMessage: 1,
          updateProfile: 0,
          editMessage: 0,
          createGroup: 0,
          createOneToOneChat: 0,
        ));
    AppStorages.secKey = secretKey;
    AppStorages.myUserId = userId;
    AppStorages.myFirstName.value = firstName;
    AppStorages.myLastName.value = lastName;
    AppStorages.myEmail.value = email;
    AppStorages.myUserName.value = userName;
    if (setUserRole != null) {
      AppStorages.myRoles = setUserRole;
    }
    AppStorages.setChatUserData(userName, setKey: SessionKeys.chatUserName);
    AppStorages.setChatUserData(userId, setKey: SessionKeys.chatUserId);
    AppStorages.setChatUserData(firstName,
        setKey: SessionKeys.chatUserFirstName);
    AppStorages.setChatUserData(lastName, setKey: SessionKeys.chatUserLastName);
    AppStorages.setChatUserData(email, setKey: SessionKeys.chatUserEmail);
    AppStorages.setChatUserData(mobile, setKey: SessionKeys.chatUserMobile);
    AppStorages.setChatUserData(profilePicture,
        setKey: SessionKeys.chatUserProfilePic);
    AppStorages.setChatUserData(fcmToken, setKey: SessionKeys.chatFcmToken);
  }

  /// ...  LOGIN USER ... ///
  static loginWithAPlusChat({
    required String userId,
    required String secretKey,
    required String fcmToken,
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFail)? onConnectionFailed,
  }) async {
    socket.connect();
    socket.onDisconnect((_) {
      debugPrint('disconnect');
      socket.io
        ..disconnect()
        ..connect();
    });
var param = {
  "userId" : userId,
  "secretKey" :secretKey};
    socket.emit("get-profile", param);
socket.on("profile-res", (data) {
  if(data['userId'] != null) {
    AppStorages.isSomethingWrong.value = false;
    AppStorages.myUserId = userId;
    AppStorages.secKey = secretKey;
    var res = data;
    AppStorages.setChatUserId(userId);
    AppStorages.setSecKey(secretKey);
    AppStorages.setChatUserData(fcmToken,
        setKey: SessionKeys.chatFcmToken);
    AppStorages.setChatUserData(res['mobile_email'],
        setKey: SessionKeys.chatUserEmail);

    AppStorages.setChatUserData(res['name'],
        setKey: SessionKeys.chatUserName);

    AppStorages.myEmail.value = res['mobile_email'];
    AppStorages.myUserName.value = res['name'];
    AppStorages.myProfilePicture.value = res['profilePicture'];

    // AppStorages.myRoles = AppStorages.getChatUserRoles();
  }else{
    AppStorages.isSomethingWrong.value = true;
    print("!!!!!");
  }
});
    // Webservice.postRequest(
    //     context: Get.context,
    //     uri: Global.updateFCMToken + userId,
    //     body: {"secretKey": secretKey, "fcmToken": fcmToken},
    //     baseUrl: Global.baseApiUrl,
    //     hasBearer: false,
    //     onSuccess: (response) {
    //       AppStorages.myUserId = userId;
    //       AppStorages.secKey = secretKey;
    //       var res = jsonDecode(response);
    //       AppStorages.setChatUserId(userId);
    //       AppStorages.setSecKey(secretKey);
    //       AppStorages.setChatUserData(fcmToken,
    //           setKey: SessionKeys.chatFcmToken);
    //       AppStorages.setChatUserData(res['item']['email'],
    //           setKey: SessionKeys.chatUserEmail);
    //       AppStorages.setChatUserData(res['item']['firstName'],
    //           setKey: SessionKeys.chatUserFirstName);
    //       AppStorages.setChatUserData(res['item']['lastName'],
    //           setKey: SessionKeys.chatUserLastName);
    //       AppStorages.setChatUserData(res['item']['username'],
    //           setKey: SessionKeys.chatUserName);
    //
    //       AppStorages.myEmail.value = res['item']['email'];
    //       AppStorages.myFirstName.value = res['item']['firstName'];
    //       AppStorages.myLastName.value = res['item']['lastName'];
    //       AppStorages.myUserName.value = res['item']['username'];
    //       AppStorages.myProfilePicture.value = res['item']['profilePicture'];
    //
    //       // AppStorages.myRoles = AppStorages.getChatUserRoles();
    //
    //       if (onSuccess != null) {
    //         onSuccess(response);
    //       }
    //     },
    //     onFailure: (response) {
    //       onError!(response);
    //     },
    //     onConnectionFailed: (response) {
    //       onConnectionFailed!(response);
    //     });
  }

  /// ... SIGN UP (Add User) ... ///
  static signUpWithAPlusChat({
    required String userId,
    required String fcmToken,
    required String userName,
    required String secretKey,
    required String email,
    firstName,
    lastName,
    @required mobile,
    profilePicture,
    SetUserRole? setUserRole,
    @required Function(dynamic value)? onSuccess,
    @required Function(dynamic value)? onError,
    @required Function(dynamic value)? onConnectionFailed,
  }) {
    var body = {
      "userId": userId,
      "fcmToken": fcmToken,
      "username": userName,
      "firstName": firstName,
      "lastName": lastName,
      "email": email,
      "phone": mobile,
      "secretKey": secretKey,
      "profilePicture": profilePicture,
      "role": setUserRole ??
          SetUserRole(
            deleteMessage: 0,
            deleteChat: 0,
            sendMessage: 1,
            updateProfile: 0,
            editMessage: 0,
            createGroup: 0,
            createOneToOneChat: 0,
          ),
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.createUser,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          AppStorages.setChatUserRoles(setUserRole ??
              SetUserRole(
                deleteMessage: 0,
                deleteChat: 0,
                sendMessage: 1,
                updateProfile: 0,
                editMessage: 0,
                createGroup: 0,
                createOneToOneChat: 0,
              ));
          AppStorages.secKey = secretKey;
          AppStorages.myUserId = userId;
          AppStorages.myFirstName.value = firstName;
          AppStorages.myLastName.value = lastName;
          AppStorages.myEmail.value = email;
          AppStorages.myUserName.value = userName;
          if (setUserRole != null) {
            AppStorages.myRoles = setUserRole;
          }
          AppStorages.setChatUserData(userName,
              setKey: SessionKeys.chatUserName);
          AppStorages.setChatUserData(userId, setKey: SessionKeys.chatUserId);
          AppStorages.setChatUserData(firstName,
              setKey: SessionKeys.chatUserFirstName);
          AppStorages.setChatUserData(lastName,
              setKey: SessionKeys.chatUserLastName);
          AppStorages.setChatUserData(email, setKey: SessionKeys.chatUserEmail);
          AppStorages.setChatUserData(mobile,
              setKey: SessionKeys.chatUserMobile);
          AppStorages.setChatUserData(profilePicture,
              setKey: SessionKeys.chatUserProfilePic);
          AppStorages.setChatUserData(fcmToken,
              setKey: SessionKeys.chatFcmToken);

          if (response != null) {
            // AppStorages.myUserName.value =
            //     AppStorages.getChatUserData(getKey: SessionKeys.chatUserName);
            // AppStorages.myFirstName.value = AppStorages.getChatUserData(
            //     getKey: SessionKeys.chatUserFirstName);
            // AppStorages.myLastName.value = AppStorages.getChatUserData(
            //     getKey: SessionKeys.chatUserLastName);
            // AppStorages.myEmail.value =
            //     AppStorages.getChatUserData(getKey: SessionKeys.chatUserEmail);
            // AppStorages.myMobile.value =
            //     AppStorages.getChatUserData(getKey: SessionKeys.chatUserMobile);
            // AppStorages.myProfilePicture = AppStorages.getChatUserData(
            //     getKey: SessionKeys.chatUserProfilePic);
            // AppStorages.chatFCMToken.value =
            //     AppStorages.getChatUserData(getKey: SessionKeys.chatFcmToken);
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// ... LOGOUT FROM APLUS CHAT ... ///
  static logoutFromAPlusChat({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic onConnectionFailed)? connectionFailed,
  }) {
    Webservice.postRequest(
        context: Get.context,
        uri: Global.updateFCMToken + AppStorages.myUserId,
        body: {"secretKey": AppStorages.secKey, "fcmToken": ""},
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          AppStorages.box.erase();
          AppStorages.myUserId = "";
          AppStorages.secKey = "";
          socket.disconnect();
          if (response != null) {
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          connectionFailed!(response);
        });
  }

  /// ... CREATE A GROUP (ONE TO ONE) ... ///
  static connectWithFriend({required opponentUserId}) {
    var body = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId,
      "email": AppStorages.myEmail.value,
      "members": [
        AppStorages.myUserId.toString(),
        opponentUserId.toString(),
      ],
      "isGroup": false,
      "groupImage": "",
      "name": "",
      "pinGroupForAll": 1,
      "groupPermission": [
        AppStorages.getChatUserRoles(),
      ]
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.createGroup,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {},
        onFailure: (response) {},
        onConnectionFailed: (response) {});
  }

  static openConversation({
    required opponentUserId,
    required BuildContext context,
  }) {
    var body = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId,
      "email": AppStorages.myEmail.value,
      "members": [
        AppStorages.myUserId.toString(),
        opponentUserId.toString(),
      ],
      "isGroup": false,
      "groupImage": "",
      "name": "",
      "pinGroupForAll": 1,
      "groupPermission": [
        AppStorages.getChatUserRoles(),
      ]
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.createGroup,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          var res = jsonDecode(response);
          if (response != null) {
            Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => MessageView(
                      groupId: res['groupId'],
                      isOnlineCall: true,
                      //opponentUserId: opponentUserId,
                      isGroup: false)),
            );
          }
        },
        onFailure: (response) {},
        onConnectionFailed: (response) {});
  }

  /// ... OPEN CONVERSATION FROM NOTIFICATION ... ///
  static openConversationFromNotification({
    required groupId,
    required isGroup,
    required BuildContext context,
  }) {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => MessageView(
                  groupId: groupId,
                  isGroup: isGroup,
                  isOnlineCall: true,
                userBlocked:0
                )));
  }

  /// ...  UPDATE CHAT USER PROFILE ... ///
  static updateChatUserProfile({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFailed)? onConnectionFailed,
    profilePic,
    firstName,
    lastName,
    userName,
    profilePicture,
  }) {
    var body = {
      "secretKey": AppStorages.secKey,
      "firstName": firstName,
      "lastName": lastName,
      "username": userName,
      "profilePicture": profilePicture,
      "userId": AppStorages.myUserId,
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.updateProfile + AppStorages.myUserId,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            AppStorages.setChatUserData(userName,
                setKey: SessionKeys.chatUserName);
            AppStorages.setChatUserData(firstName,
                setKey: SessionKeys.chatUserFirstName);
            AppStorages.setChatUserData(lastName,
                setKey: SessionKeys.chatUserLastName);
            AppStorages.setChatUserData(profilePicture,
                setKey: SessionKeys.chatUserProfilePic);
            AppStorages.myUserName.value =
                AppStorages.getChatUserData(getKey: SessionKeys.chatUserName);
            AppStorages.myFirstName.value = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserFirstName);
            AppStorages.myLastName.value = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserLastName);
            AppStorages.myProfilePicture = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserProfilePic);
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }






  /// block User ... ///
  static blockUser({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFailed)? onConnectionFailed,
    var userID,
  }) {
    var body = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId,
      "blockUserId": userID,




    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.blockUser,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// GET ALL UNREAD COUNT ... ///
  static getCount(Function(dynamic) unreadCount) {
    var params = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId
    };
    socket.emit('user-unread-count', params);
    socket.on('user-unread-count-res-${AppStorages.myUserId}', (data) {
      unreadCount(data);
    });

  }



  static chatListView() {
    return  ChatsListView();
  }
}
