import 'dart:convert';
import 'dart:io';
import 'package:aplus_chat/core/services/stream_socket.dart';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/images.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/helper/utilites.dart';
import 'package:aplus_chat/model/group_list_model.dart';
import 'package:aplus_chat/view/contact_list_view.dart';
import 'package:aplus_chat/view/message_view.dart';
import 'package:aplus_chat/view/profile_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:aplus_chat/view/widgets/themes/cometchat_theme.dart';
import 'package:aplus_chat/view/widgets/themes/palette.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:socket_io_client/socket_io_client.dart';

class ChatsListView extends StatefulWidget {
  CometChatTheme? theme;
   ChatsListView({this.theme});

  @override
  ChatsListViewState createState() => ChatsListViewState();
}

class ChatsListViewState extends State<ChatsListView> {
  TextEditingController searchController = TextEditingController();
  List snap = [];
  CometChatTheme theme = cometChatTheme;

  String searchText = '';
  var hasCreateGRPPermission = false.obs;
  var hasOneToOneChatPermission = false.obs;
  @override
  void initState() {
    theme = widget.theme ?? cometChatTheme;
    if (socket.disconnected) {
      socket.connect();
      socket.onDisconnect((_) {
        socket.io
          ..disconnect()
          ..connect();
      });
    }
    getUserProfileApi();
    getGroupList();

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () => willPop(),
      child: Scaffold(
        backgroundColor: theme.palette.mode == PaletteThemeModes.light?Colors.white:Colors.white12,
        body: SafeArea(
          child: Obx(()=>AppStorages.isSomethingWrong.value?InkWell(
    onTap: () {

    },
    child: const Text("Something wrong")) :  Column(
            children: [
              appBar(),
          Padding(
                  padding: const EdgeInsets.only(left: 20, right: 20, top: 10),
                  child: TextField(
                    controller: searchController,
                    onChanged: (val) {
                      setState(() {
                        searchText = val;
                      });
                    },
                    style: TextStyle(color: theme.palette.getAccent800()),
                    decoration: InputDecoration(
                      hintText: "Search chat",
                      hintStyle: CustomTextStyles.medium(
                          fontSize: 14.0, fontColor: Colors.grey),
                      filled: true,

                      fillColor: theme.palette.primary.light,
                      border: searchCommonBorderStyle,
                      enabledBorder: searchCommonBorderStyle,
                      disabledBorder: searchCommonBorderStyle,
                      focusedBorder: searchCommonBorderStyle,
                      focusedErrorBorder: searchCommonBorderStyle,
                      errorBorder: searchCommonBorderStyle,
                      contentPadding: const EdgeInsets.only(left: 6, right: 6),
                      prefixIcon:  Padding(
                          padding: const EdgeInsetsDirectional.only(start: 6.0),
                          child: Icon(Icons.search,color: theme.palette.accent.light,)),
                    ),
                  )),
               Padding(
                padding: EdgeInsets.only(top: 4),
                child: Divider(
                  color: theme.palette.getAccent400(),
                  thickness: 1,
                ),
              ),
              Expanded(
                child: StreamBuilder(
                    stream: chatListSoket.getResponse,
                    builder: (BuildContext context,
                        AsyncSnapshot<dynamic> snapshot) {
                      if (snapshot.connectionState == ConnectionState.waiting) {
                        return const Center(child: CircularProgressIndicator());
                      }
                      if (snapshot.hasData) {
                        var groupListModel =
                            groupListModelFromJson(snapshot.data);

                        if (searchText.isNotEmpty) {
                          groupListModel = groupListModel.where((element) {
                            return element.groupName
                                .toString()
                                .toLowerCase()
                                .contains(searchText.toLowerCase());
                          }).toList();
                        }
                        return groupListModel.isEmpty
                            ? const Center(child: Text("Data not found"))
                            : ListView.separated(
                                padding:
                                    const EdgeInsets.symmetric(vertical: 10),
                                itemBuilder: (context, index) {
                                  return chatItem(groupListModel[index], index);
                                },
                                separatorBuilder: (context, index) {
                                  return  Padding(
                                    padding:
                                        EdgeInsets.only(top: 10, bottom: 10),
                                    child: Divider(
                                      color:  theme.palette.getAccent400(),
                                      thickness: 1.0,
                                      height: 1.0,
                                    ),
                                  );
                                },
                                itemCount: groupListModel.length);
                      } else {
                        return Container();
                      }
                    }),
              ),
            ],
          ),
        ),
      ),
    ));
  }

  opponentUser(element) {
    var opponentUser = {};
    element["users"].forEach((ele) {
      if (ele["userId"] != AppStorages.myUserId) {
        opponentUser = ele;
      }
    });
    return opponentUser;
  }

  willPop() {
    offline();
    Navigator.pop(context);
    socket.off("get-group-list-res");
  }

  Widget appBar() {
    return Container(
      color: theme.palette.getAccent200(),
      padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 10),
      child: Obx(
        () => Row(
          children: [
            InkWell(
                onTap: () {
                  willPop();
                },
                child: Icon(Platform.isAndroid?Icons.arrow_back:Icons.arrow_back_ios,color: theme.palette.getAccent800(),)),
            const SizedBox(width: 8),
            GestureDetector(
              onTap: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(builder: (context) =>  ProfileView(theme: theme,)),
                );
              },
              child: Obx(() => Common.profileNetworkImage(
                  borderRadius: 60.0,
                  height: 40.0,
                  width: 40.0,
                  imageUrl: AppStorages.myProfilePicture.value)),
            ),
            const SizedBox(width: 8),
            Expanded(
              child: Obx(() => Text(
                AppStorages.myUserName.value,
                style: CustomTextStyles.medium(
                    fontSize: 16.0, fontColor:  theme.palette.getAccent800()),
              )),
            ),
            hasCreateGRPPermission.value
                ? IconButton(
                    icon: Icon(Icons.group_add,size: 25,color: theme.palette.getAccent800(),),
                    onPressed: () {
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                            builder: (context) =>
                                ContactListView(forMakeGroup: true,theme: theme,)),
                      );
                    },
                  )
                : const SizedBox.shrink(),
            hasOneToOneChatPermission.value
                ? IconButton(
                    icon: Icon(Icons.person_add,size: 25,color: theme.palette.getAccent800(),),
                    onPressed: () {
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                            builder: (context) =>
                                ContactListView(forMakeGroup: false,theme: theme,)),
                      );
                    },
                  )
                : const SizedBox.shrink(),
          ],
        ),
      ),
    );
  }

  var searchCommonBorderStyle = OutlineInputBorder(
      borderSide: const BorderSide(
        color: Color(0xffe6e6e6),
      ),
      borderRadius: BorderRadius.circular(16));

  chatItem(GroupListModel data, index) {
    return Padding(
      padding: const EdgeInsets.only(left: 12, right: 12),
      child: InkWell(
        onTap: () {
          Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => MessageView(
                      groupId: data.groupId,
                      title: data.groupName,
                      //opponentUserId: data.opponentUserId,
                      theme: theme,
                      isGroup: data.isGroup,userBlocked: data.isBlocked)));
        },
        child: Row(
          children: [
            Common.profileNetworkImage(
                borderRadius: 56.0,
                height: 56.0,
                width: 56.0,
                placeHolder: data.isGroup!
                    ? ImageResource.groupPlaceHolder
                    : ImageResource.profilePlaceHolder,
                imageUrl: data.imagePath),
            const SizedBox(width: 10),
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Row(
                    children: [
                      Expanded(
                        child: Text(
                          data.groupName!,
                          style: CustomTextStyles.semiBold(
                            fontSize: 16.0,
                            fontColor:  theme.palette.getAccent800(),
                          ),
                        ),
                      ),
                      const SizedBox(width: 10),
                      data.latestTime == null
                          ? const SizedBox.shrink()
                          : Text(
                              Utilities.convertTimeStampToDate(
                                  data.latestTime!.seconds ?? ""),
                              style: CustomTextStyles.normal(
                                  fontSize: 10.0, fontColor: theme.palette.mode == PaletteThemeModes.light?Colors.black38:Colors.white38),
                            )
                    ],
                  ),
                  const SizedBox(height: 4),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Row(
                        children: [
                          data.msgType == "text" ||
                                  data.recentMsg == null ||
                                  data.msgType == ""
                              ? const SizedBox.shrink()
                              : Image.asset(
                                  data.msgType == "audio"
                                      ? ImageResource.audioIc
                                      : data.msgType == "video"
                                          ? ImageResource.videoIc
                                          : data.msgType == "image"
                                              ? ImageResource.imageIc
                                              : ImageResource.documentIc,
                                  height: 20,
                                  width: 20,
                                  package: "aplus_chat",
                                ),
                          Container(
                            width: MediaQuery.of(context).size.width/2,
                            child: Text(
                              data.msgType! == ""
                                  ? " Start your conversation"
                                  : data.msgType! == "text"
                                      ? data.recentMsg!
                                      : " ${data.msgType!}",
                              style: CustomTextStyles.normal(
                                fontSize: 12.0,
                                fontColor: theme.palette.getAccent500(),
                              ),
                              maxLines: 1,
                              overflow: TextOverflow.ellipsis,
                            ),
                          ),
                        ],
                      ),

                      data.unreadCount == 0
                          ? const SizedBox.shrink()
                          : Container(
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(20.0),
                                  color: theme.palette.getAccent800()),
                              padding: const EdgeInsets.only(
                                  left: 4.5, right: 4, top: 1.5, bottom: 1),
                              child: Text(
                                "${data.unreadCount}",
                                style: CustomTextStyles.medium(
                                  fontSize: 10.0,
                                  fontColor:  theme.palette.mode == PaletteThemeModes.light?Colors.black38:Colors.white38,
                                ),
                              ),
                            )

                      // CircleAvatar(
                      //         backgroundColor: Colors.red,
                      //         radius: 10,
                      //         child: Text(
                      //           "100",
                      //           style: CustomTextStyles.medium(
                      //             fontSize: 8.0,
                      //             fontColor: Colors.white,
                      //           ),
                      //         ),
                      //       )
                    ],
                  )
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  getUserProfileApi() async {
    var body = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId
    };
    Webservice.postRequest(
      uri: Global.getUserDetails,
      body: body,
      baseUrl: Global.baseApiUrl,
      hasBearer: false,
      onSuccess: (response) {
        var res = jsonDecode(response);

        AppStorages.setChatUserData(res['item']['username'],
            setKey: SessionKeys.chatUserName);
        AppStorages.setChatUserData(res['item']['userId'],
            setKey: SessionKeys.chatUserId);
        AppStorages.setChatUserData(res['item']['firstName'],
            setKey: SessionKeys.chatUserFirstName);
        AppStorages.setChatUserData(res['item']['lastName'],
            setKey: SessionKeys.chatUserLastName);
        AppStorages.setChatUserData(res['item']['email'],
            setKey: SessionKeys.chatUserEmail);

        AppStorages.setChatUserData(res['item']['profilePicture'],
            setKey: SessionKeys.chatUserProfilePic);

        // AppStorages.setSecKey(res['item']['secretKey']);
        AppStorages.myUserName.value = res['item']['username'];
        AppStorages.myFirstName.value = res['item']['firstName'];
        AppStorages.myLastName.value = res['item']['lastName'];
        AppStorages.myEmail.value = res['item']['email'];
        AppStorages.myUserId = res['item']['userId'];
        AppStorages.myProfilePicture.value = res['item']['profilePicture'];
        AppStorages.chatFCMToken.value =
            AppStorages.getChatUserData(getKey: SessionKeys.chatFcmToken);

        // AppStorages.secKey = res['item']['secretKey'];
        hasCreateGRPPermission.value = res['item']['role']['createGroup'] == 1;
        hasOneToOneChatPermission.value =
            res['item']['role']['createOneToOneChat'] == 1;
      },
      onFailure: (v) {},
    );
  }
}
