
import 'dart:io';

import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/colors.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/model/contact_list_model.dart';
import 'package:aplus_chat/model/group_detail_model.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:aplus_chat/view/create_group_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:aplus_chat/view/widgets/themes/cometchat_theme.dart';
import 'package:aplus_chat/view/widgets/themes/palette.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../core/services/stream_socket.dart';

// ignore: must_be_immutable
class ContactListView extends StatefulWidget {
  ContactListView({
    Key? key,
    this.forMakeGroup = false,
    this.forAddMember = false,
    this.participentList,
    this.groupId,
    this.theme,
  }) : super(key: key);
  bool forMakeGroup = false;
  bool forAddMember = false;
  // ignore: prefer_typing_uninitialized_variables
  List<ParticipantUsers>? participentList = [];
  CometChatTheme? theme;
  // ignore: prefer_typing_uninitialized_variables
  var groupId;

  @override
  ContactListViewState createState() => ContactListViewState();
}

class ContactListViewState extends State<ContactListView> {
  RxList<UserList> usersList = <UserList>[].obs;
  RxList<UserList> searchUserList = <UserList>[].obs;
  String searchText = '';
  var loadData = false.obs;
  @override
  void initState() {
    loadData.value = true;
    getUsersApi();
    super.initState();
  }

  Widget appBar() {
    return Container(
      color: widget.theme!.palette.getAccent200(),
      padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 10),
      child: Row(
          children: [
            InkWell(
                onTap: () {
                Navigator.of(context).pop();
                },
                child: Icon(Platform.isAndroid?Icons.arrow_back:Icons.arrow_back_ios,color: widget.theme!.palette.getAccent800(),)),
            const SizedBox(width: 8),
            Expanded(
              child:  Text(
                widget.forAddMember
                    ? "Add Member"
                    : widget.forMakeGroup
                    ? "Create Group"
                    : "Contact list",
                style: CustomTextStyles.medium(
                    fontSize: 16.0, fontColor:  widget.theme!.palette.getAccent800()),
              ),
            ),
          ],
        ),

    );
  }

  TextEditingController searchController = TextEditingController();
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      bottomNavigationBar: widget.forMakeGroup
          ? SafeArea(
              child: GestureDetector(
                onTap: addMemberLoader.value
                    ? null
                    : () {
                        var selectedUserIds = [];

                        selectedUserIds = [];
                        for (var element in usersList) {
                          if (element.isSelected.value) {
                            selectedUserIds.add(element.userId);
                          }
                        }

                        if (selectedUserIds.isEmpty) {
                          Common.getSnackToast(
                            context,
                            title: 'Error',
                            message: "Please select user",
                          );
                        } else {
                          if (widget.forAddMember) {
                            addMemberLoader.value = true;
                            addMemberApi(otherUserId: selectedUserIds,contexts: context);
                          } else {
                            Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => CreateGroup(
                                  selectedUserIds: selectedUserIds,
                                  theme: widget.theme,
                                ),
                              ),
                            );
                          }
                        }
                      },
                child: Obx(
                  () => Container(
                    height: 50,
                    margin: const EdgeInsets.only(top: 16, left: 20, right: 20,bottom: 20),
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(8.0),
                      color: widget.theme!.palette.getAccent(),
                    ),
                    child: Center(
                      child: addMemberLoader.value
                          ? const CircularProgressIndicator(color: Colors.white)
                          : Text(
                              widget.forAddMember ? "Add" : "Next",
                              style: CustomTextStyles.semiBold(
                                  fontSize: 18.0, fontColor: Colors.white),
                            ),
                    ),
                  ),
                ),
              ),
            )
          : const SizedBox(),
      body: SafeArea(
        child: Container(
          color: widget.theme!.palette.mode == PaletteThemeModes.light?Colors.white:Colors.black54,
          child: Column(
            children: [
              appBar(),
              Padding(
                padding:
                    const EdgeInsets.only(left: 20, right: 20, top: 8, bottom: 8),
                child: TextField(
                  controller: searchController,
                  onChanged: (val) {
                    List<UserList> dummySearchList = <UserList>[];
                    dummySearchList.addAll(searchUserList);
                    if (val.isNotEmpty) {
                      List<UserList> dummyListData = <UserList>[];
                      for (var item in dummySearchList) {
                        if (item.username.toString().toLowerCase().contains(val)) {
                          dummyListData.add(item);
                        }
                      }
                      usersList.clear();
                      usersList.addAll(dummyListData);
                      usersList.refresh();
                      return;
                    } else {
                      usersList.clear();
                      usersList.addAll(searchUserList);
                    }
                  },
                  decoration: InputDecoration(
                    hintText: "Search user",
                    hintStyle: CustomTextStyles.medium(
                        fontSize: 14.0, fontColor: Colors.black38),
                    filled: true,
                    fillColor: widget.theme!.palette.getAccent200(),
                    border: searchCommonBorderStyle,
                    enabledBorder: searchCommonBorderStyle,
                    disabledBorder: searchCommonBorderStyle,
                    focusedBorder: searchCommonBorderStyle,
                    focusedErrorBorder: searchCommonBorderStyle,
                    errorBorder: searchCommonBorderStyle,
                    contentPadding: const EdgeInsets.only(left: 6, right: 6),
                    prefixIcon: const Padding(
                        padding: EdgeInsetsDirectional.only(start: 6.0),
                        child: Icon(Icons.search)),
                  ),
                ),
              ),
              Expanded(
                child: Obx(
                  () => loadData.value
                      ? const Center(
                          child: CircularProgressIndicator(),
                        )
                      : usersList.isEmpty
                          ? const Center(
                              child: Text("No Contact found"),
                            )
                          : ListView.separated(
                              padding: const EdgeInsets.only(
                                  left: 20, right: 20, top: 8),
                              itemBuilder: (context, index) {
                                return contactItem(usersList[index]);
                              },
                              separatorBuilder: (_, index) {
                                return const SizedBox(height: 8);
                              },
                              itemCount: usersList.length,
                            ),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  var searchCommonBorderStyle = const OutlineInputBorder(
    borderSide: BorderSide(
      color: Color(0xffE6E6E6),
    ),
  );

  Widget contactItem(UserList users) {

    return InkWell(
      onTap: () {
        if (widget.forMakeGroup) {
          users.isSelected.value = !users.isSelected.value;
        } else {
          connectUser(otherUserId: users.userId);
        }
      },
      child: Row(
        children: [
          widget.forMakeGroup
              ? Obx(() => users.isSelected.value
                  ?  Icon(Icons.check_box,color: widget.theme!.palette.mode == PaletteThemeModes.light?Colors.black:Colors.white,)
                  :  Icon(Icons.check_box_outline_blank,color: widget.theme!.palette.mode == PaletteThemeModes.light?Colors.black:Colors.white,))
              : const SizedBox.shrink(),
          Padding(
            padding:
                EdgeInsets.only(left: widget.forMakeGroup ? 10 : 0, right: 10),
            child: Common.profileNetworkImage(
              borderRadius: 48.0,
              height: 48.0,
              width: 48.0,
              imageUrl: users.profilePicture,
            ),
          ),
          Text(
            "${users.username}",
            style: CustomTextStyles.medium(
                fontSize: 18.0, fontColor: widget.theme!.palette.getAccent800()),
          )
        ],
      ),
    );
  }

  var connectLoader = false.obs;
  connectUser({otherUserId}) {
    var body = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId,

      "members": [
        AppStorages.myUserId,
        otherUserId,
      ],
      "isGroup": false,
      "groupImage": "",
      "name": "",
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.createGroup,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            Navigator.pushReplacement(
                context,
                MaterialPageRoute(
                    builder: (BuildContext context) =>  ChatsListView()));
          }
        },
        onFailure: (response) {},
        onConnectionFailed: (response) {});
  }

  var addMemberLoader = false.obs;
  addMemberApi({List? otherUserId,BuildContext? contexts}) {
    var user = [];
    var  groupId =  widget.groupId;
    for(int i=0;i<widget.participentList!.length;i++){
      user.add(widget.participentList![i].userId);
    }


    var body = {
      "addMembersArr": otherUserId,
      "groupId": groupId,
      "secretKey": AppStorages.secKey,
      "members":user+otherUserId!,
      "viewBy":user+otherUserId,
    };
    print(body);

    socket.emit("add-member", body);
    socket.on("add-member-res", (data) {

      addMemberLoader.value = false;

      Navigator.pop(contexts!, "true");
    });



    // Webservice.postRequest(
    //     context: Get.context,
    //     uri: Global.addMember,
    //     body: body,
    //     baseUrl: Global.baseApiUrl,
    //     hasBearer: false,
    //     onSuccess: (response) {
    //       addMemberLoader.value = false;
    //       if (response != null) {
    //         Navigator.pop(context, "true");
    //       }
    //     },
    //     onFailure: (response) {
    //       addMemberLoader.value = false;
    //     },
    //     onConnectionFailed: (response) {
    //       addMemberLoader.value = false;
    //     });
  }

  getUsersApi() {
var body = {"userId":AppStorages.myUserId,"secretKey":AppStorages.secKey};

    socket.emit("user-list", body);
    socket.on("user-list-res", (data) {
      var usersListMode = userListModelFromJson(data);
      usersList.addAll(usersListMode.list!);
      searchUserList.addAll(usersListMode.list!);
      loadData.value = false;
      if (widget.forAddMember) {
        if (widget.participentList != null) {
          if (usersList.isNotEmpty) {

            List<ParticipantUsers> participentList = widget.participentList!;
            for (var val in participentList) {
              usersList
                  .removeWhere((element) => element.userId == val.userId);
              usersList.refresh();
            }
          }
        }
      }


    });


  }
}
