// ignore_for_file: must_be_immutable

import 'dart:convert';
import 'dart:io';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/keyboard_avoider.dart';
import 'package:aplus_chat/helper/colors.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/images.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:aplus_chat/view/widgets/themes/cometchat_theme.dart';
import 'package:aplus_chat/view/widgets/themes/palette.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class CreateGroup extends StatefulWidget {

  CreateGroup({Key? key, this.selectedUserIds,this.theme}) : super(key: key);

  // ignore: prefer_typing_uninitialized_variables
  CometChatTheme? theme;
  var selectedUserIds;
  @override
  CreateGroupState createState() => CreateGroupState();
}

class CreateGroupState extends State<CreateGroup> {
  TextEditingController groupNameController = TextEditingController();
  // ignore: prefer_typing_uninitialized_variables
  var imageBytes;
  var groupImage = File("").obs;

  var selectedUserIds = [];
  var imageUploadLoader = false.obs;

  @override
  void initState() {
    super.initState();
    selectedUserIds = [];
    selectedUserIds = widget.selectedUserIds;
    selectedUserIds.add(AppStorages.myUserId);
  }

  Widget appBar() {
    return Container(
      color: widget.theme!.palette.getAccent200(),
      padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 10),
      child: Row(
        children: [
          InkWell(
              onTap: () {
                Navigator.of(context).pop();
              },
              child: Icon(Platform.isAndroid?Icons.arrow_back:Icons.arrow_back_ios,color: widget.theme!.palette.getAccent800(),)),
          const SizedBox(width: 8),
          Expanded(
            child:  Text(
              "Create Group",
              style: CustomTextStyles.medium(
                  fontSize: 16.0, fontColor:  widget.theme!.palette.getAccent800()),
            ),
          ),
        ],
      ),

    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Container(
          color: widget.theme!.palette.mode == PaletteThemeModes.light?Colors.white:Colors.black54,
          child: Column(
            children: [
              appBar(),
              const SizedBox(height: 10),
              InkWell(
                borderRadius: BorderRadius.circular(200),
                onTap: () async {
                  imageUploadLoader.value = true;
                  File pickedFile;
                  FilePickerResult? result = await FilePicker.platform.pickFiles(
                    type: FileType.media,
                    withData: true,
                  );
                  pickedFile = File(result!.paths[0].toString());
                  groupImage.value = pickedFile;
                  uploadImageApi(
                      file: result.paths[0],
                      fileName:
                          result.paths[0].toString().split("/").last.toString());
                },
                child: Stack(
                  alignment: Alignment.center,
                  children: [
                    Obx(
                      () => !groupImage.value.isAbsolute
                          ? Container(
                              height: 200,
                              width: 200,
                              decoration: const BoxDecoration(
                                color: Colors.black12,
                                shape: BoxShape.circle,
                                image: DecorationImage(
                                  image: ExactAssetImage(
                                      ImageResource.groupPlaceHolder),
                                  fit: BoxFit.cover,
                                ),
                              ),
                            )
                          : ClipRRect(
                              borderRadius: BorderRadius.circular(200),
                              child: Container(
                                color: Colors.black12,
                                child: Image.file(
                                  File(
                                    groupImage.value.path,
                                  ),
                                  height: 200.0,
                                  width: 200.0,
                                  fit: BoxFit.cover,
                                ),
                              ),
                            ),
                    ),
                    imageUploadLoader.value
                        ? const Center(
                      child: CircularProgressIndicator(),
                    )
                        : const Icon(Icons.camera_alt_outlined, color: Colors.white),
                  ],
                ),
              ),
              const SizedBox(height: 10),
              Expanded(
                child: KeyboardAvoider(
                  autoScroll: true,
                  child: Column(
                    children: [
                      Padding(
                        padding: const EdgeInsets.only(
                            left: 20, right: 20, top: 8, bottom: 8),
                        child: TextField(
                          controller: groupNameController,
                          onChanged: (val) {},
                          style: TextStyle(color:widget.theme!.palette.getAccent800()),
                          decoration: InputDecoration(
                            hintText: "Enter group name",
                            hintStyle: CustomTextStyles.medium(
                                fontSize: 14.0, fontColor: const Color(0xffBFBFC4)),
                            filled: true,
                            fillColor: const Color(0xffEEEEEF),
                            border: commonBorderStyle,
                            enabledBorder: commonBorderStyle,
                            disabledBorder: commonBorderStyle,
                            focusedBorder: commonBorderStyle,
                            focusedErrorBorder: commonBorderStyle,
                            errorBorder: commonBorderStyle,
                            contentPadding:
                                const EdgeInsets.only(left: 6, right: 6),
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              )
            ],
          ),
        ),
      ),
      bottomNavigationBar: SafeArea(
        child: GestureDetector(
            onTap: () {
              if (groupNameController.text.isEmpty) {
                Common.getSnackToast(context,
                    title: "Error", message: "Please enter group name");
              } else {
                var createGRPParam = {
                  "secretKey": AppStorages.secKey,
                  "userId": AppStorages.myUserId,
                  //"email": AppStorages.myEmail.value,
                  "members": selectedUserIds,
                  "isGroup": true,
                  "groupImage": selectedGRPImageUrl,
                  "name": groupNameController.text,
                };
                Webservice.postRequest(
                    context: Get.context,
                    uri: Global.createGroup,
                    body: createGRPParam,
                    baseUrl: Global.baseApiUrl,
                    hasBearer: false,
                    onSuccess: (response) {
                      if (response != null) {
                        Navigator.pushReplacement(
                            context,
                            MaterialPageRoute(
                                builder: (BuildContext context) =>
                                     ChatsListView()));
                      }
                    },
                    onFailure: (response) {},
                    onConnectionFailed: (response) {});
              }
            },
            child:Container(
                      height: 50,
                      margin: const EdgeInsets.only(
                          top: 16, left: 20, right: 20, bottom: 10),
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(8.0),
                        color: widget.theme!.palette.getAccent800(),
                      ),
                      child: Center(
                        child: Text(
                          "Create Group",
                          style: CustomTextStyles.medium(
                              fontSize: 18.0, fontColor: Colors.white),
                        ),
                      ),
                    ),
            )),

    );
  }

  // Common border for textField
  var commonBorderStyle = const OutlineInputBorder(
    borderSide: BorderSide(color: Color(0xffE6E6E6), width: 1.0),
  );
  var selectedGRPImageUrl = "";
  var selectGRPImageName = "";

  uploadImageApi({contentType, file, fileName, pickdFile, fileType}) async {


    var body = {
      "secretKey": AppStorages.secKey.toString(),
      "userId":AppStorages.myUserId.toString(),
      "groupId":"",
      "type":"image",
      "isChat":"0",
    };
    Webservice.uploadImage(
      uri: Global.uploadFileNew,
      body: body,
      hasBearer: true,
      onSuccess: (response) {

        var res = jsonDecode(response);
        selectedGRPImageUrl = res['file'];
        imageUploadLoader.value = false;
        selectGRPImageName = res['fileName'];
      },
      onFailure: (v) {
        imageUploadLoader.value = false;
      }, file: File(file), fileType: 'selectFile',
    );

  }
}
