// ignore_for_file: prefer_typing_uninitialized_variables

import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:ui';
import 'package:aplus_chat/core/services/stream_socket.dart';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/images.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/helper/utilites.dart';
import 'package:aplus_chat/model/conversation_list_model.dart';
import 'package:aplus_chat/view/contact_info_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:aplus_chat/view/widgets/group_info_view.dart';
import 'package:aplus_chat/view/widgets/image_view_dialog.dart';
import 'package:aplus_chat/view/widgets/themes/cometchat_theme.dart';
import 'package:aplus_chat/view/widgets/themes/palette.dart';
import 'package:aplus_chat/view/widgets/video_player.dart';
import 'package:better_open_file/better_open_file.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:grouped_list/grouped_list.dart';
import 'package:image_picker/image_picker.dart';
import 'package:scroll_to_index/scroll_to_index.dart';
import 'package:swipe_to/swipe_to.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:video_thumbnail/video_thumbnail.dart';
import 'widgets/audio_play_dialog.dart';

// ignore: must_be_immutable
class MessageView extends StatefulWidget {

  MessageView(
      {Key? key, this.groupId, this.isGroup, this.isOnlineCall,this.theme, this.title,this.userBlocked
      //this.opponentUserId,
      })
      : super(key: key);
  CometChatTheme? theme ;
  String? groupId;
  bool? isGroup;
  bool? isOnlineCall = false;
  String? title = "";
  int? userBlocked;

  // String? opponentUserId;
  @override
  MessageViewState createState() => MessageViewState();
}

class MessageViewState extends State<MessageView>
    with SingleTickerProviderStateMixin {
  var info;
  RxList<Message> conversationList = <Message>[].obs;
  final ImagePicker picker = ImagePicker();
  var hasMore = false;
  var startType = true;
  GroupData? groupData;
  var otherUserTyping = false.obs;
  var otherTyingUserName = "".obs;
  var isUserOnline = false.obs;
  var hasActionPermission = false.obs;
  var hasSendMessagePermission = true;
  RxList<PopupMenuEntry<int>> actionList = <PopupMenuEntry<int>>[].obs;
  late Animation<Color?> animation;
  late AnimationController controller;
  var groupId = "";
  var groupName = "".obs, groupImage = "".obs;
  var opponentUserId = "";
  var isOnlineCall;

  @override
  void initState() {
    isOnlineCall = widget.isOnlineCall ?? false;
    widget.title = widget.title ?? "";
widget.theme = widget.theme??cometChatTheme;
    groupName.value = widget.title!;
     if (widget.userBlocked == 0) {
    onlineUser();
    }
    receivedMessages(success: (data) {
      Message messages = messagesFromJson(data);
      if (messages.sentBy != AppStorages.myUserId) {
        conversationList.insert(0, messages);
      } else {
        conversationList.replaceRange(0, 1, [messages]);
      }
    });

    groupId = widget.groupId!;
    typingResSocket(success: (val) {
      otherUserTyping.value = val['isTyping'] == "true";
      otherTyingUserName.value = val['name'];
    });
    userActiveStatus(success: (val) {
      isUserOnline.value = val['isOnline'];
    });
    getConversation(
        groupId: groupId,
        userId: AppStorages.myUserId,
        startAt: 0,
        onSuccess: (val) {
          ConversationListModel conversationListModel =
              conversationListModelFromJson(val);
          conversationList.addAll(conversationListModel.messages!.reversed);
          hasMore = conversationListModel.hasMore!;
          groupData = conversationListModel.groupData!;

          if (groupData!.groupName != "") {
            groupName.value = groupData!.groupName!;
            groupImage.value = groupData!.imagePath!;
            opponentUserId = groupData!.opponentUserId!;
            isUserOnline.value = groupData!.onlineStatus!;
          }
          if (groupData != null &&
              groupData!.userPermission!.permission != null) {
            if (widget.isGroup!) {
              hasSendMessagePermission =
                  groupData!.userPermission!.permission!.sendMessage == 1;

              if (groupData!.userPermission!.permission!.clearChat == 1) {
                actionList.add(PopupMenuItem(
                  onTap: () {
                    cleanPopUp((){
                      clearChat(
                          userId: AppStorages.myUserId,
                          groupId: groupId,
                          onSuccess: () {
                            conversationList.clear();
                          });
                    },"Are You Sure Want To Clean ?");

                  },
                  value: 3,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Text(
                        "Clear chat",
                        style: CustomTextStyles.medium(
                            fontSize: 16.0, fontColor: Colors.black),
                      ),
                      const Icon(Icons.message_outlined),
                    ],
                  ),
                ));
              }
              if (groupData!.userPermission!.permission!.deleteChat == 1) {
                actionList.add(PopupMenuItem(
                  value: 2,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Text(
                        "Delete chat",
                        style: CustomTextStyles.medium(
                            fontSize: 16.0, fontColor: Colors.black),
                      ),
                      const Icon(Icons.delete),
                    ],
                  ),
                  onTap: () {
                    cleanPopUp((){
                      if (groupData!.isGroup == true) {
                        deleteGroup(
                            userId: AppStorages.myUserId,
                            groupId: groupId,
                            context: context);
                      } else {
                        deleteChat(
                            userId: AppStorages.myUserId,
                            groupId: groupId,
                            context: context);
                      }
                    },"Are You Sure Want To Delete ?");




                  },
                ));
              }
              hasActionPermission.value = actionList.isNotEmpty ? true : false;
            } else {
              actionList.add(PopupMenuItem(
                value: 2,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Text(
                      "Delete chat",
                      style: CustomTextStyles.medium(
                          fontSize: 16.0, fontColor: Colors.black),
                    ),
                    const Icon(Icons.delete),
                  ],
                ),
                onTap: () {
                  cleanPopUp((){
                    if (groupData!.isGroup == true) {
                      deleteGroup(
                          userId: AppStorages.myUserId,
                          groupId: groupId,
                          context: context);
                    } else {
                      deleteChat(
                          userId: AppStorages.myUserId,
                          groupId: groupId,
                          context: context);
                    }
                  },"Are You Sure Want To Delete ?");


                },
              ));
              actionList.add(PopupMenuItem(
                onTap: () {
                  cleanPopUp((){
                    clearChat(
                        userId: AppStorages.myUserId,
                        groupId: groupId,
                        onSuccess: () {
                          conversationList.clear();
                        });
                  },"Are You Sure Want To Clean ?");



                },
                value: 3,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Text(
                      "Clear chat",
                      style: CustomTextStyles.medium(
                          fontSize: 16.0, fontColor: Colors.black),
                    ),
                    const Icon(Icons.message_outlined),
                  ],
                ),
              ));
              hasActionPermission.value = true;
            }
          }
        });

    autoScrollController = AutoScrollController(keepScrollOffset: true);
    autoScrollController.addListener(scroll);

    /// BLINK ANIMATION ////

    controller = AnimationController(
      duration: const Duration(milliseconds: 600),
      reverseDuration: const Duration(seconds: 1),
      vsync: this,
    );
    final CurvedAnimation curve =
        CurvedAnimation(parent: controller, curve: Curves.linear);
    animation = ColorTween(
            begin: const Color(0xff8EB4C9).withOpacity(0.8), end: Colors.blue)
        .animate(curve);
    // Keep the animation going forever once it is started
    animation.addStatusListener((status) {
      // Reverse the animation after it has been completed
      if (status == AnimationStatus.completed) {
        controller.reverse();
      }
    });
    super.initState();
  }

  @override
  dispose() {
    controller.dispose();
    super.dispose();
  }


  cleanPopUp(onTap,title){
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: const Text('Alert'),
          content: Text(title),
          actions: <Widget>[
            MaterialButton(
              child: const Text("NO"),
              onPressed: () {
                //Put your code here which you want to execute on No button click.
                Navigator.of(context).pop();
              },
            ),
            MaterialButton(
              onPressed: onTap,
              child: const Text("YES"),
            ),



          ],
        );
      },
    );

  }

  scroll() {
    if (autoScrollController.position.maxScrollExtent ==
        autoScrollController.position.pixels) {
      if (hasMore) {
        var convoParam = {
          "groupId": groupId,
          "userId": AppStorages.myUserId,
          "startAt": conversationList[conversationList.length - 1].time! - 1,
          "secretKey": AppStorages.secKey,
        };
        socket.emit('get-chat', convoParam);
      }
    } else {
      if (autoScrollController.position.pixels == 0) {
        debugPrint("TOP");
      }
    }
  }

  AutoScrollController autoScrollController = AutoScrollController();

  willPop() {
    var da = isOnlineCall;
    if (da!) {
      offline();
    }
    socket.off("typing-res");
    socket.off("online-status");
    socket.off("receive-message");
    leaveConvo(groupId: groupId, userId: AppStorages.myUserId);

    Navigator.pop(context);
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: () => willPop(),
        child: Scaffold(
          body: SafeArea(
            top: false,
            bottom: true,
            child: Container(
              decoration:  BoxDecoration(
                color: widget.theme!.palette.primary.light,
                image:  DecorationImage(
                  image: ExactAssetImage(widget.theme!.palette.mode ==PaletteThemeModes.light? ImageResource.chatBg:ImageResource.darkChatBg,
                      package: "aplus_chat"),
                  fit: BoxFit.cover,
                ),
              ),
              child: Column(
                children: [
                  appBar(),
                  Expanded(
                    child: Obx(
                      () => GroupedListView<dynamic, String>(
                        // ignore: invalid_use_of_protected_member
                        elements: conversationList.value,
                        groupBy: (element) => Utilities.convertTimeStampToDate(
                            element.timeMilliSeconds.seconds,
                            formate: 'yyyy-MM-dd'),
                        order: GroupedListOrder.DESC,
                        reverse: true,
                        controller: autoScrollController,
                        stickyHeaderBackgroundColor: Colors.transparent,
                        floatingHeader: true,
                        separator: const SizedBox(height: 0),
                        useStickyGroupSeparators: true,
                        groupSeparatorBuilder: (value) {
                          return groupSapeatorItem(value);
                        },
                        indexedItemBuilder: (context, item, index) {
                          final CurvedAnimation curve = CurvedAnimation(
                              parent: controller, curve: Curves.linear);
                          animation = conversationList[index].sentBy ==
                                  AppStorages.myUserId
                              ? ColorTween(
                                      begin: const Color(0xff8EB4C9)
                                          .withOpacity(0.8),
                                      end: Colors.white)
                                  .animate(curve)
                              : ColorTween(
                                      begin: Colors.white,
                                      end: const Color(0xff8EB4C9)
                                          .withOpacity(0.8))
                                  .animate(curve);
                          return AutoScrollTag(
                              key: ValueKey(index),
                              controller: autoScrollController,
                              index: index,
                              child: AnimatedBuilder(
                                  animation: animation,
                                  builder:
                                      (BuildContext context, Widget? child) {
                                    return SwipeTo(
                                      onRightSwipe:
                                          conversationList[index].sentBy ==
                                                  AppStorages.myUserId
                                              ? (v){}
                                              : (v) {
                                                  repliedMessage.value =
                                                      conversationList[index];
                                                  isReplied.value = true;
                                                  FocusScope.of(context)
                                                      .requestFocus(focusNode);
                                                },
                                      onLeftSwipe:
                                          conversationList[index].sentBy ==
                                                  AppStorages.myUserId
                                              ? (v) {
                                                  repliedMessage.value =
                                                      conversationList[index];
                                                  isReplied.value = true;
                                                  FocusScope.of(context)
                                                      .requestFocus(focusNode);
                                                }
                                              : (v){},
                                      child: InkWell(
                                        onLongPress: () {},
                                        child: Padding(
                                          padding: EdgeInsets.only(
                                            left: conversationList[index]
                                                        .sentBy ==
                                                    AppStorages.myUserId
                                                ? 100.0
                                                : 10,
                                            right: conversationList[index]
                                                        .sentBy ==
                                                    AppStorages.myUserId
                                                ? 10
                                                : 80,
                                            top: 10,
                                            bottom: 10,
                                          ),
                                          child: Row(
                                            mainAxisSize: MainAxisSize.min,
                                            mainAxisAlignment:
                                                conversationList[index]
                                                            .sentBy ==
                                                        AppStorages.myUserId
                                                    ? MainAxisAlignment.end
                                                    : MainAxisAlignment.start,
                                            crossAxisAlignment:
                                                CrossAxisAlignment.end,
                                            children: [
                                              const SizedBox(width: 6),
                                              Expanded(
                                                child: Column(
                                                  crossAxisAlignment:
                                                      conversationList[index]
                                                                  .sentBy ==
                                                              AppStorages
                                                                  .myUserId
                                                          ? CrossAxisAlignment
                                                              .end
                                                          : CrossAxisAlignment
                                                              .start,
                                                  children: [
                                                    Container(
                                                      padding:
                                                          const EdgeInsets.all(
                                                              8),
                                                      decoration: BoxDecoration(
                                                        color: conversationList[
                                                                    index]
                                                                .startAnimation
                                                                .value
                                                            ? animation.value
                                                            : conversationList[
                                                                            index]
                                                                        .sentBy ==
                                                                    AppStorages
                                                                        .myUserId
                                                                ? widget.theme!.palette.getAccent100()
                                                                : widget.theme!.palette.getAccent200(),
                                                        borderRadius:
                                                            BorderRadius
                                                                .circular(6),
                                                      ),
                                                      child: Padding(
                                                        padding:
                                                            const EdgeInsets
                                                                .all(0.0),
                                                        child: Column(
                                                          crossAxisAlignment:
                                                              CrossAxisAlignment
                                                                  .start,
                                                          children: [
                                                            widget.isGroup!
                                                                ? conversationList[index]
                                                                            .sentBy !=
                                                                        AppStorages
                                                                            .myUserId
                                                                    ? Padding(
                                                                        padding: const EdgeInsets
                                                                            .only(
                                                                            bottom:
                                                                                8),
                                                                        child:
                                                                            Text(
                                                                          conversationList[index]
                                                                              .senderName!,
                                                                          style: CustomTextStyles.bold(
                                                                              fontSize: 14.0,
                                                                              fontColor: Colors.blue),
                                                                        ),
                                                                      )
                                                                    : const SizedBox
                                                                        .shrink()
                                                                : const SizedBox
                                                                    .shrink(),
                                                            messageType(
                                                                conversationList[
                                                                    index],
                                                                index),
                                                          ],
                                                        ),
                                                      ),
                                                    ),
                                                    const SizedBox(height: 8),
                                                    Text(
                                                      Utilities
                                                          .convertTimeStampToTime(
                                                              conversationList[
                                                                      index]
                                                                  .timeMilliSeconds!
                                                                  .seconds),
                                                      textAlign:
                                                          conversationList[
                                                                          index]
                                                                      .sentBy ==
                                                                  AppStorages
                                                                      .myUserId
                                                              ? TextAlign.right
                                                              : TextAlign.left,
                                                      style: CustomTextStyles
                                                          .normal(
                                                              fontSize: 8.0,
                                                              fontColor:
                                                                  Colors.black),
                                                    ),
                                                  ],
                                                ),
                                              ),
                                            ],
                                          ),
                                        ),
                                      ),
                                    );
                                  }));
                        },
                      ),
                    ),
                  ),
                  widget.userBlocked == 0?
                  messageSendView():SizedBox(),
                ],
              ),
            ),
          ),
        ));
  }

  groupSapeatorItem(value) {
    String date = ChatDateFormatter().groupChatDateHeader(value);
    return SizedBox(
      height: 40,
      child: Align(
        alignment: Alignment.center,
        child: Container(
          padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 10),
          decoration: const BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(5.0)),
          ),
          child: Text(
            date,
            style: CustomTextStyles.normal(
                fontSize: 10.0, fontColor: Colors.black),
            textAlign: TextAlign.center,
          ),
        ),
      ),
    );
  }

  Widget appBar() {
    return SafeArea(
      child: Container(
        color: widget.theme!.palette.getAccent200(),
        padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 0),
        child: Row(
          children: [
            const SizedBox(width: 10),
            InkWell(
                onTap: () {
                  willPop();
                },
                child:  Icon(Platform.isAndroid?Icons.arrow_back:Icons.arrow_back_ios,color: widget.theme!.palette.getAccent800(),)),
            const SizedBox(width: 8),
            GestureDetector(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => !widget.isGroup!
                          ? ContactInfoView(userId: opponentUserId,theme: widget.theme,)
                          : GroupInfoView(groupId: groupId),
                    ));
              },
              child: Obx(() => Common.profileNetworkImage(
                    borderRadius: 60.0,
                    height: 40.0,
                    width: 40.0,
                    placeHolder: widget.isGroup!
                        ? ImageResource.groupPlaceHolder
                        : ImageResource.profilePlaceHolder,
                    imageUrl: groupImage.value,
                  )),
            ),
            const SizedBox(width: 8),
            Obx(
              () => Expanded(
                child: GestureDetector(
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => !widget.isGroup!
                            ? ContactInfoView(userId: opponentUserId,theme: widget.theme,)
                            : GroupInfoView(groupId: groupId),
                      ),
                    );
                  },
                  child: isUserOnline.value || otherUserTyping.value
                      ? Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            Text(groupName.value,
                                style: CustomTextStyles.semiBold(
                                    fontSize: 18.0, fontColor: widget.theme!.palette.accent.light)),
                            otherUserTyping.value
                                ? Text(
                                    otherUserTyping.value
                                        ? widget.isGroup!
                                            ? "${otherTyingUserName.value} Typing ..."
                                            : " Typing..."
                                        : "",
                                    style: CustomTextStyles.medium(
                                        fontSize: 14.0, fontColor: Colors.blue),
                                  )
                                : isUserOnline.value && !widget.isGroup!
                                    ? Text(
                                        " Online",
                                        style: CustomTextStyles.medium(
                                            fontSize: 14.0,
                                            fontColor: Colors.green),
                                      )
                                    : const SizedBox.shrink(),
                          ],
                        )
                      : Text(groupName.value,
                          style: CustomTextStyles.semiBold(
                              fontSize: 18.0, fontColor: widget.theme!.palette.accent.light)),
                ),
              ),
            ),
            Obx(() => !hasActionPermission.value
                ? const SizedBox.shrink()
                : PopupMenuButton<int>(
                    icon:  Icon(
                      Icons.more_vert_outlined,
                      color: widget.theme!.palette.accent.light,
                    ),
                    // ignore: invalid_use_of_protected_member
                    itemBuilder: (context) => actionList.value,
                    offset: const Offset(0, 50),
                    color: widget.theme!.palette.primary.light,
                    elevation: 2,
                  )),
          ],
        ),
      ),
    );
  }

  var isReplied = false.obs;
  Rx<Message> repliedMessage = Message().obs;

  messageType(Message chat, index) {
    var fileName;

    if (chat.type == "text") {
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          chat.replyMsgId == ""
              ? const SizedBox.shrink()
              : InkWell(
                  onTap: () async {
                    for (var element in conversationList) {
                      if (element.startAnimation.value) {
                        element.startAnimation.value = false;
                      }
                    }
                    var selcIndex = conversationList
                        .indexWhere((item) => item.msgId == chat.replyMsgId);

                    if (selcIndex != -1) {
                      await autoScrollController.scrollToIndex(selcIndex,
                          preferPosition: AutoScrollPosition.end);
                      conversationList[selcIndex].startAnimation.value = true;
                      controller.forward();
                    }
                  },
                  child: Container(
                    margin: const EdgeInsets.only(right: 20, bottom: 8),
                    decoration: BoxDecoration(
                      color: widget.theme!.palette.getAccent100(),
                      borderRadius: BorderRadius.circular(8),
                    ),
                    child: IntrinsicHeight(
                      child: Row(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Container(
                            width: 4.0,
                            decoration:  BoxDecoration(
                              color: widget.theme!.palette.getAccent800(),
                              borderRadius: const BorderRadius.only(
                                topLeft: Radius.circular(8.0),
                                bottomLeft: Radius.circular(8.0),
                              ),
                            ),
                          ),
                          Expanded(
                              child: Padding(
                            padding: const EdgeInsets.only(
                                top: 12, bottom: 12, left: 12),
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Text(
                                  chat.replyUserId == AppStorages.myUserId
                                      ? "You"
                                      : chat.replyUser!,
                                  style: CustomTextStyles.medium(
                                      fontSize: 14.0,
                                      fontColor:  widget.theme!.palette.getAccent800()),
                                  overflow: TextOverflow.ellipsis,
                                ),
                                const SizedBox(height: 4),
                                Text(
                                   chat.replyMsg!??"",
                                  style: CustomTextStyles.normal(
                                      fontSize: 12.0, fontColor: widget.theme!.palette.getAccent600()),
                                  overflow: TextOverflow.ellipsis,
                                )
                              ],
                            ),
                          )),
                          chat.replyMsgType != "text"
                              ? ClipRRect(
                                  borderRadius: BorderRadius.circular(4.0),
                                  child:   chat.replyMsgType ==
                                      "document"
                                      ? Padding(
                                    padding: const EdgeInsets.all(8.0),
                                    child: Center(
                                      child: Container(
                                        height: 35,
                                        width: 35,
                                        decoration: BoxDecoration(
                                          color: widget.theme!.palette.getAccent800(),
                                          borderRadius:
                                          BorderRadius.circular(40),
                                        ),
                                        child: const Icon(
                                          Icons.attach_file,
                                          color: Colors.white,
                                          size: 20,
                                        ),
                                      ),
                                    ),
                                  )
                                      : chat.replyMsgType =="audio"?Padding(
                                    padding: const EdgeInsets.all(8.0),
                                    child: Center(
                                      child: Container(
                                        height: 35,
                                        width: 35,
                                        decoration: BoxDecoration(
                                          color: widget.theme!.palette.getAccent800(),
                                          borderRadius:
                                          BorderRadius.circular(40),
                                        ),
                                        child: const Icon(
                                          Icons.audiotrack,
                                          color: Colors.white,
                                          size: 20,
                                        ),
                                      ),
                                    ),
                                  ):
                                  chat.replyMsg!.isURL
                                          ? Hero(
                                              tag: 'htg_0',
                                              child: CachedNetworkImage(
                                                fadeInDuration: const Duration(
                                                    milliseconds: 250),
                                                height: 60,
                                                width: 52,
                                                imageUrl: chat.replyMsg!,
                                                errorWidget:
                                                    (context, url, error) =>
                                                        Opacity(
                                                  opacity: 0.9,
                                                  child: ClipRRect(
                                                    borderRadius:
                                                        BorderRadius.circular(
                                                            30.0),
                                                    child: Image.asset(
                                                      ImageResource
                                                          .groupPlaceHolder,
                                                      package: "aplus_chat",
                                                    ),
                                                  ),
                                                ),
                                                fit: BoxFit.cover,
                                                placeholder:
                                                    ((context, String s) =>
                                                        Image.asset(
                                                          ImageResource
                                                              .groupPlaceHolder,
                                                          height: 40,
                                                          width: 40,
                                                          package: "aplus_chat",
                                                        )),
                                              ))
                                          : Image.file(
                                              File(
                                                chat.replyMsg!,
                                              ),
                                              height: 60,
                                              width: 52,
                                            ))
                              : const SizedBox.shrink(),
                        ],
                      ),
                    ),
                  ),
                ),
          Text(chat.message!,
              style: CustomTextStyles.medium(
                  fontSize: 14.0, fontColor: widget.theme!.palette.getAccent()),
              textAlign: chat.sentBy == AppStorages.myUserId
                  ? TextAlign.right
                  : TextAlign.left),
        ],
      );
    } else if (chat.type == "audio") {
      return InkWell(
        onTap: () async {
          Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => Player(
                      audioFile: chat.filePath, fileName: chat.fileName)));
        },
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                Flexible(
                  fit: FlexFit.loose,
                  child: Column(
                    children: [
                      Text(
                      chat.fileName.toString(),
                        style: CustomTextStyles.medium(
                            fontSize: 12.0,
                            fontColor: widget.theme!.palette.getAccent800(),
                            isUnderLine: true),
                      ),
                    ],
                  ),
                ),
                const SizedBox(width: 5),
                Container(
                  height: 35,
                  width: 35,
                  decoration: BoxDecoration(
                    color: widget.theme!.palette.getAccent800(),
                    borderRadius: BorderRadius.circular(40),
                  ),
                  child: const Icon(
                    Icons.audiotrack,
                    color: Colors.white,
                    size: 20,
                  ),
                ),
              ],
            ),
          ],
        ),
      );
    } else if (chat.type == "document") {
      return InkWell(
        onTap: () async {
          if (chat.filePath!.isURL) {
            if (!await launchUrl(
              Uri.parse(chat.filePath!),
              mode: LaunchMode.externalApplication,
            )) {
              throw 'Could not launch ${chat.filePath}';
            }
          } else {
            debugPrint("OPEN FILE");
            OpenFile.open(chat.filePath).then((value) {
              if (value.message.contains("No APP found to open this file")) {
                Common.getSnackToast(context,
                    title: "Error", message: value.message);
              }
            }, onError: (e) {
              Common.getSnackToast(context,
                  title: "Error", message: e.toString());
            });
          }
        },
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                Flexible(
                  fit: FlexFit.loose,
                  child: Column(
                    children: [
                      Text(
                        chat.fileName.toString(),
                        style: CustomTextStyles.medium(
                            fontSize: 12.0,
                            fontColor: widget.theme!.palette.getAccent800(),
                            isUnderLine: true),
                      ),
                    ],
                  ),
                ),
                const SizedBox(width: 5),
                Container(
                  height: 35,
                  width: 35,
                  decoration: BoxDecoration(
                    color: widget.theme!.palette.getAccent800(),
                    borderRadius: BorderRadius.circular(40),
                  ),
                  child: const Icon(
                    Icons.attach_file,
                    color: Colors.white,
                    size: 20,
                  ),
                ),
              ],
            ),
          ],
        ),
      );
    } else if (chat.type == "image") {
      return InkWell(
        onTap: () {
          showDialog(
              context: context,
              barrierDismissible: false,
              builder: (BuildContext context) {
                return ImagePreview(image: chat.filePath);
              });
        },
        child: ClipRRect(
          borderRadius: BorderRadius.circular(10),
          child: Container(
              height: 200,
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10),
              ),
              child: chat.filePath!.isURL
                  ? Hero(
                      tag: 'htg_$index',
                      child: CachedNetworkImage(
                        fadeInDuration: const Duration(milliseconds: 250),
                        imageUrl: chat.filePath!,
                        width: 140,
                        errorWidget: (context, url, error) => Opacity(
                          opacity: 0.9,
                          child: ClipRRect(
                            borderRadius: BorderRadius.circular(30.0),
                            child: Image.asset(
                              ImageResource.groupPlaceHolder,
                              package: "aplus_chat",
                            ),
                          ),
                        ),
                        fit: BoxFit.cover,
                        placeholder: ((context, String s) => Image.asset(
                              ImageResource.groupPlaceHolder,
                              // height: 40,
                              // width: 40,
                              package: "aplus_chat",
                            )),
                      ))
                  : Stack(alignment: Alignment.center, children: [
                      Image.file(File(chat.filePath!)),
                      chat.fileUploadLoader.value
                          ? const SizedBox(
                              // height: 40,
                              // width: 40,
                              child: CircularProgressIndicator())
                          : const SizedBox.shrink(),
                    ])),
        ),
      );
    } else if (chat.type == "video") {
      return InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => VideoPlayerView(
                videoFile: chat.filePath,
                isVideoUrl: chat.filePath!.isURL,
              ),
            ),
          );
        },
        child: ClipRRect(
          borderRadius: BorderRadius.circular(10),
          child: Stack(
            alignment: AlignmentDirectional.center,
            children: [
              Container(
                height: 200,
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                ),
                child: chat.thumbnailPath!.isURL
                    ? Hero(
                        tag: 'htg_$index',
                        child: CachedNetworkImage(
                          fadeInDuration: const Duration(milliseconds: 250),
                          imageUrl: chat.thumbnailPath!,
                          errorWidget: (context, url, error) => Opacity(
                            opacity: 0.9,
                            child: ClipRRect(
                              borderRadius: BorderRadius.circular(30.0),
                              child: Image.asset(
                                ImageResource.groupPlaceHolder,
                                package: "aplus_chat",
                              ),
                            ),
                          ),
                          fit: BoxFit.cover,
                          placeholder: ((context, String s) => Image.asset(
                                ImageResource.groupPlaceHolder,
                                height: 40,
                                width: 40,
                                package: "aplus_chat",
                              )),
                        ),
                      )
                    : Stack(alignment: Alignment.center, children: [
                        Image.file(
                          File(chat.thumbnailPath!),
                        ),
                        chat.fileUploadLoader.value
                            ? const SizedBox(child: CircularProgressIndicator())
                            : const SizedBox.shrink(),
                      ]),
              ),
              const Icon(Icons.play_circle)
            ],
          ),
        ),
      );
    }
  }

  messageSendView() {
    return Container(
      padding: const EdgeInsets.only(top: 6, bottom: 6),
      decoration:  BoxDecoration(color: widget.theme!.palette.getAccent200()),
      child: Column(
        children: [
          Obx(() => isReplied.value
              ? Container(
                  margin: const EdgeInsets.only(left: 20, right: 20, bottom: 8),
                  decoration: BoxDecoration(
                    color: widget.theme!.palette.getAccent100(),
                    borderRadius: BorderRadius.circular(8),
                  ),
                  child: IntrinsicHeight(
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Container(
                          width: 4.0,
                          decoration:  BoxDecoration(
                             color: widget.theme!.palette.getAccent800(),
                            borderRadius: BorderRadius.only(
                              topLeft: Radius.circular(8.0),
                              bottomLeft: Radius.circular(8.0),
                            ),
                          ),
                        ),
                        Expanded(
                            child: Padding(
                          padding: const EdgeInsets.only(
                              top: 12, bottom: 12, left: 12),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Obx(() => Text(
                                    repliedMessage.value.sentBy ==
                                            AppStorages.myUserId
                                        ? "You"
                                        : repliedMessage.value.senderName!,
                                    style: CustomTextStyles.medium(
                                        fontSize: 14.0,
                                        fontColor: widget.theme!.palette.getAccent800()),
                                    overflow: TextOverflow.ellipsis,
                                  )),
                              const SizedBox(height: 4),
                              Obx(() => Text(
                                    repliedMessage.value.type == "text"
                                        ? repliedMessage.value.message!
                                        : repliedMessage.value.fileName!,
                                    style: CustomTextStyles.normal(
                                        fontSize: 12.0,
                                        fontColor: widget.theme!.palette.getAccent600()),
                                    overflow: TextOverflow.ellipsis,
                                  ))
                            ],
                          ),
                        )),
                        repliedMessage.value.type != "text"
                            ? Stack(
                                alignment: Alignment.topRight,
                                children: [
                                  ClipRRect(
                                      borderRadius: BorderRadius.circular(4.0),
                                      child: repliedMessage.value.type ==
                                              "document"
                                          ? Padding(
                                            padding: const EdgeInsets.all(8.0),
                                            child: Center(
                                              child: Container(
                                                  height: 35,
                                                  width: 35,
                                                  decoration: BoxDecoration(
                                                    color: widget.theme!.palette.getAccent800(),
                                                    borderRadius:
                                                        BorderRadius.circular(40),
                                                  ),
                                                  child: const Icon(
                                                    Icons.attach_file,
                                                    color: Colors.white,
                                                    size: 20,
                                                  ),
                                                ),
                                            ),
                                          )
                                          :repliedMessage.value.type =="audio"?Padding(
                                        padding: const EdgeInsets.all(8.0),
                                        child: Center(
                                          child: Container(
                                            height: 35,
                                            width: 35,
                                            decoration: BoxDecoration(
                                              color: widget.theme!.palette.getAccent800(),
                                              borderRadius:
                                              BorderRadius.circular(40),
                                            ),
                                            child: const Icon(
                                              Icons.audiotrack,
                                              color: Colors.white,
                                              size: 20,
                                            ),
                                          ),
                                        ),
                                      ) :repliedMessage.value.filePath!.isURL
                                              ? Hero(
                                                  tag: 'htg_0',
                                                  child: CachedNetworkImage(
                                                    fadeInDuration:
                                                        const Duration(
                                                            milliseconds: 250),
                                                    height: 60,
                                                    width: 52,
                                                    imageUrl: repliedMessage
                                                                .value.type ==
                                                            "video"
                                                        ? repliedMessage.value
                                                            .thumbnailPath!
                                                        : repliedMessage
                                                            .value.filePath!,
                                                    errorWidget:
                                                        (context, url, error) =>
                                                            Opacity(
                                                      opacity: 0.9,
                                                      child: ClipRRect(
                                                        borderRadius:
                                                            BorderRadius
                                                                .circular(30.0),
                                                        child: Image.asset(
                                                          ImageResource
                                                              .groupPlaceHolder,
                                                          package: "aplus_chat",
                                                        ),
                                                      ),
                                                    ),
                                                    fit: BoxFit.cover,
                                                    placeholder: ((context,
                                                            String s) =>
                                                        Image.asset(
                                                          ImageResource
                                                              .groupPlaceHolder,
                                                          height: 40,
                                                          width: 40,
                                                          package: "aplus_chat",
                                                        )),
                                                  ))
                                              : Image.file(
                                                  File(
                                                    repliedMessage.value.type ==
                                                            "video"
                                                        ? repliedMessage.value
                                                            .thumbnailPath!
                                                        : repliedMessage
                                                            .value.filePath!,
                                                  ),
                                                  height: 60,
                                                  width: 52,
                                                )),
                                  InkWell(
                                    onTap: () {
                                      isReplied.value = false;
                                    },
                                    child: Container(
                                      margin: const EdgeInsets.only(
                                          top: 2, right: 4),
                                      padding: EdgeInsets.all(1.0),
                                      decoration:  BoxDecoration(
                                          shape: BoxShape.circle,
                                          color: widget.theme!.palette.getAccent800()),
                                      child: const Icon(
                                        Icons.clear,color: Colors.white,
                                        size: 12,
                                      ),
                                    ),
                                  ),
                                ],
                              )
                            : IconButton(
                                onPressed: () {
                                  isReplied.value = false;
                                },
                                icon: const Icon(Icons.clear),
                              ),
                      ],
                    ),
                  ),
                )
              : const SizedBox.shrink()),
          Row(
            children: [
              IconButton(
                  onPressed: () {
                    showAttachDialog();
                  },
                  icon:  Icon(Icons.attach_file,color: widget.theme!.palette.getAccent800(),)),
              Expanded(
                  child: TextField(
                // readOnly: true,
                controller: messageSendController,
                onChanged: (text) async {
                  if (!isTyping.value) {
                    startType = true;
                    isTyping.value = true;
                    typingSocket(
                        groupId: groupId,
                        userId: AppStorages.myUserId,
                        secKey: AppStorages.secKey,
                        name: groupData!.userName,
                        isTyping: "true");
                    Future.delayed(const Duration(seconds: 2)).whenComplete(() {
                      startType = false;
                      isTyping.value = false;
                      typingSocket(
                          groupId: groupId,
                          userId: AppStorages.myUserId,
                          secKey: AppStorages.secKey,
                          name: groupData!.userName,
                          isTyping: "false");
                    });
                  }

                  if (text.isEmpty) {
                    sendButtonEnable.value = false;
                  } else {
                    sendButtonEnable.value = true;
                  }
                },

                maxLines: 7,
                minLines: 1,
                style: const TextStyle(
                  // fontFamily: Styles.FontNameRegular,
                  color: Colors.black,
                ),
                focusNode: focusNode,
                textInputAction: TextInputAction.newline,
                decoration: InputDecoration(
                  fillColor: Colors.white,
                  filled: true,
                  contentPadding: const EdgeInsets.only(left: 10, right: 10),
                  hintText: "Type a message",
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(6),
                    borderSide: BorderSide.none,
                  ),
                ),
              )),
              hasSendMessagePermission
                  ? Obx(
                      () => IconButton(
                        onPressed: sendButtonEnable.value
                            ? () {
                                var sendMsgParam = {
                                  "messageObj": {
                                    "message": messageSendController.text,
                                    "type": "text",
                                    "sentBy": AppStorages.myUserId,
                                    "senderName": groupData!.userName,
                                    "replyUser": isReplied.value
                                        ? repliedMessage.value.senderName
                                        : "",
                                    "replyMsg": isReplied.value
                                        ? repliedMessage.value.type == "text"
                                            ? repliedMessage.value.message
                                            : repliedMessage.value.type ==
                                                    "video"
                                                ? repliedMessage
                                                    .value.thumbnailPath
                                                : repliedMessage.value.type ==
                                                        "image"
                                                    ? repliedMessage
                                                        .value.filePath
                                                    : repliedMessage.value.fileName
                                        : "",
                                    "replyMsgId": isReplied.value
                                        ? repliedMessage.value.msgId
                                        : "",
                                    "replyUserId": isReplied.value
                                        ? repliedMessage.value.sentBy
                                        : "",
                                    "replyMsgType": isReplied.value
                                        ? repliedMessage.value.type
                                        : ""
                                  },
                                  "groupId": groupId,"fileName": repliedMessage.value.fileName,
                                  "secretKey": AppStorages.secKey,
                                  "userId": AppStorages.myUserId
                                };
                                DateTime now = DateTime.now();
                                int timestamp = now.millisecondsSinceEpoch;

                                conversationList.insert(
                                    0,
                                    Message(
                                      time: timestamp ~/ 1000,
                                      timeMilliSeconds: TimeMilliSeconds(
                                          seconds: timestamp ~/ 1000,
                                          nanoseconds: timestamp ~/ 1000),
                                      sentBy: AppStorages.myUserId,
                                      senderName: groupData!.userName,
                                      type: "text",
                                      msgId: isReplied.value
                                          ? repliedMessage.value.msgId
                                          : "",
                                      message: messageSendController.text,
                                      contentType: "",
                                      replyUser: isReplied.value
                                          ? repliedMessage.value.senderName
                                          : "",
                                      replyMsg: isReplied.value
                                          ? repliedMessage.value.type == "text"
                                              ? repliedMessage.value.message
                                              : repliedMessage.value.type ==
                                                      "video"
                                                  ? repliedMessage
                                                      .value.thumbnailPath
                                                  : repliedMessage.value.type ==
                                                          "image"
                                                      ? repliedMessage
                                                          .value.filePath
                                                      : repliedMessage
                                                          .value.name
                                          : "",
                                      replyMsgId: isReplied.value
                                          ? repliedMessage.value.msgId
                                          : "",
                                      replyUserId: isReplied.value
                                          ? repliedMessage.value.sentBy
                                          : "",
                                      replyMsgType: isReplied.value
                                          ? repliedMessage.value.type
                                          : "",
                                      fileName: repliedMessage.value.fileName,
                                      filePath: "",
                                      name: "",
                                    ));
                                sendMessage(sendMsgParam);
                                autoScrollController.animateTo(
                                  autoScrollController.position.minScrollExtent,
                                  duration: const Duration(milliseconds: 1600),
                                  curve: Curves.easeOut,
                                );
                                isReplied.value = false;
                                messageSendController.clear();
                                sendButtonEnable.value = false;
                              }
                            : null,
                        icon: Icon(
                          Icons.send_sharp,
                          color: sendButtonEnable.value
                              ?widget.theme!.palette.getAccent800()
                              : widget.theme!.palette.getAccent400(),
                        ),
                      ),
                    )
                  : const SizedBox.shrink()
            ],
          ),
        ],
      ),
    );
  }

  TextEditingController messageSendController = TextEditingController();
  var sendButtonEnable = false.obs;
  var isTyping = false.obs;
  var focusNode = FocusNode();

  void showAttachDialog() {
    var alert = Align(
      alignment: Alignment.bottomCenter,
      child: BackdropFilter(
        filter: ImageFilter.blur(sigmaX: 3, sigmaY: 3),
        child: Padding(
          padding: const EdgeInsets.all(8),
          child: Container(
            width: MediaQuery.of(context).size.width / 1.2,
            margin: const EdgeInsets.only(bottom: 30, left: 12, right: 12),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(10),
              color: Colors.white,
            ),
            child: Material(
              color: Colors.transparent,
              borderRadius: BorderRadius.circular(10),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Align(
                    alignment: Alignment.topRight,
                    child: IconButton(
                      icon: const Icon(Icons.clear),
                      onPressed: () => Navigator.pop(context),
                    ),
                  ),
                  dialogTile(
                      text: 'Send Attachment',
                      fontWeight: FontWeight.bold,
                      fontSize: 18),
                  saperator(),
                  dialogTile(
                      text: 'Gallery',
                      callback: () {
                        pickMedia(false, false, false);
                      }),
                  saperator(),
                  dialogTile(
                      text: 'Camera',
                      callback: () {
                        pickMedia(true, false, false);
                      }),
                  saperator(),
                  dialogTile(
                      text: 'Document',
                      callback: () {
                        pickMedia(false, true, false);
                      }),
                  saperator(),
                  dialogTile(
                      text: 'Audio',
                      callback: () {
                        pickMedia(false, false, true);
                      }),
                ],
              ),
            ),
          ),
        ),
      ),
    );

    showGeneralDialog(
      barrierLabel: "Label",
      barrierDismissible: true,
      barrierColor: Colors.black.withOpacity(0.5),
      transitionDuration: const Duration(milliseconds: 400),
      context: context,
      pageBuilder: (context, anim1, anim2) {
        return alert;
      },
      transitionBuilder: (context, anim1, anim2, child) {
        return SlideTransition(
          position: Tween(begin: const Offset(0, 1), end: const Offset(0, 0))
              .animate(anim1),
          child: child,
        );
      },
    );
  }

  dialogTile(
      {required String text,
      Function? callback,
      fontWeight = FontWeight.normal,
      double fontSize = 16}) {
    return InkWell(
      onTap: () {
        if (callback != null) {
          Navigator.pop(context);
          callback();
        }
      },
      child: Container(
        width: MediaQuery.of(context).size.width,
        alignment: Alignment.center,
        child: Padding(
          padding: const EdgeInsets.only(
              right: 12.0, left: 12.0, top: 16, bottom: 16),
          child: Text(
            text,
            style: TextStyle(
              color: Colors.black,
              fontSize: fontSize,
              fontWeight: fontWeight,
            ),
          ),
        ),
      ),
    );
  }

  saperator() {
    return Padding(
        padding: const EdgeInsets.only(left: 24, right: 24),
        child: Container(height: 1, color: Colors.grey));
  }

  Future pickMedia(bool isCamera, bool isAttachment, bool isAudio) async {
    var mediaType = "image";
    var thumbnail = "";
    DateTime now = DateTime.now();
    int timestamp = now.millisecondsSinceEpoch;
    if (isCamera) {
      final XFile? image = await picker.pickImage(
        source: ImageSource.camera,
        imageQuality: 80,
      );
      conversationList.insert(
          0,
          Message(
            time: timestamp ~/ 1000,
            timeMilliSeconds: TimeMilliSeconds(
                seconds: timestamp ~/ 1000, nanoseconds: timestamp ~/ 1000),
            sentBy: AppStorages.myUserId,
            senderName: groupData!.userName,
            type: mediaType,
            msgId: "",
            message: "",
            contentType: "",
            replyMsg: "",
            replyMsgId: "",
            replyMsgType: "",
            replyUser: "",
            replyUserId: "",
            fileName: image!.name,
            filePath: image.path,
            name: "",
            thumbnailPath: thumbnail,
          ));
      conversationList[0].fileUploadLoader.value = true;
      uploadMedia(File(image.path.toString()), mediaType, image.path);
    } else {
      FilePickerResult? result = isAttachment
          ? await FilePicker.platform.pickFiles(
              type: FileType.custom,
              allowedExtensions: ['txt', 'pdf', 'doc', 'xlsx'],
            )
          : isAudio
              ? await FilePicker.platform.pickFiles(type: FileType.audio)
              : await FilePicker.platform.pickFiles(type: FileType.media);
      if (result != null) {
        File file = File(result.files.single.path!);
        PlatformFile fileData = result.files.first;
        if (isAttachment) {
          mediaType = "document";
        } else if (isAudio) {
          mediaType = "audio";
        } else {
          if (fileData.extension == 'avi' ||
              fileData.extension == 'flv' ||
              fileData.extension == 'mkv' ||
              fileData.extension == 'mov' ||
              fileData.extension == 'mp4') {
            mediaType = "video";
            thumbnail = await VideoThumbnail.thumbnailFile(
                    video: result.paths[0].toString(),
                    imageFormat: ImageFormat.JPEG) ??
                "";
          }
        }
        conversationList.insert(
            0,
            Message(
              time: timestamp ~/ 1000,
              timeMilliSeconds: TimeMilliSeconds(
                  seconds: timestamp ~/ 1000, nanoseconds: timestamp ~/ 1000),
              sentBy: AppStorages.myUserId,
              senderName: groupData!.userName,
              type: mediaType,
              msgId: "",
              message: "",
              contentType: "",
              replyMsg: "",
              replyMsgId: "",
              replyMsgType: "",
              replyUser: "",
              replyUserId: "",
              fileName: fileData.name,
              filePath: file.path,
              name: "",
              thumbnailPath: thumbnail,
            ));
        conversationList[0].fileUploadLoader.value = true;
        uploadMedia(file, mediaType, file.path);
      }
    }
  }

  uploadMedia(File selectedFile, type, filePath) async {
    var body = {
      'secretKey': AppStorages.secKey,
      'type': type.toString(),
      "userId": AppStorages.myUserId.toString(),
      "senderName": groupData!.userName.toString(),
      "groupId": groupId.toString(),
      "isChat": "1",
    };
    Webservice.uploadImage(
        uri: Global.uploadFileNew,
        file: selectedFile,
        body: body,
        fileType: 'selectFile',
        hasBearer: false,
        onSuccess: (response) {
          conversationList[0].fileUploadLoader.value = false;
          conversationList.refresh();
          autoScrollController.animateTo(
            autoScrollController.position.minScrollExtent,
            duration: const Duration(milliseconds: 1600),
            curve: Curves.easeOut,
          );
          sendButtonEnable.value = false;
        },
        onFailure: (v) {
          conversationList[0].fileUploadLoader.value = false;
          conversationList.refresh();
        },
        onTimeout: (v) {
          conversationList[0].fileUploadLoader.value = false;
          conversationList.refresh();
        },
        onConnectionFailed: () {
          conversationList[0].fileUploadLoader.value = false;
          conversationList.refresh();
        });
  }
}
