// ignore_for_file: prefer_typing_uninitialized_variables

import 'dart:convert';
import 'dart:io';

import 'package:aplus_chat/core/services/stream_socket.dart';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/colors.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/images.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/model/group_detail_model.dart';
import 'package:aplus_chat/view/contact_list_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:aplus_chat/view/widgets/themes/cometchat_theme.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class GroupInfoView extends StatefulWidget {
   GroupInfoView({Key? key, this.groupId,this.theme}) : super(key: key);
  final String? groupId;
  CometChatTheme? theme;
  @override
  State<GroupInfoView> createState() => _GroupInfoViewState();
}

class _GroupInfoViewState extends State<GroupInfoView> {
  var groupDetails;
  var groupName = "".obs;
  var participentCounts = "".obs;
  var editName = false.obs;
  RxList<ParticipantUsers> participantList = <ParticipantUsers>[].obs;
  var createdBy = "".obs;
  var groupImage = "".obs;
  var groupId = "".obs;
  var isImageEditable = false.obs;
  var loadData = true.obs;
  var hasExitGRPPermission = 0.obs;
  var hasDeleteGRPPermission = 0.obs;
  var hasRemoveMemberPermission = 0.obs;
  var hasEditGroupImagePermission = 0.obs;
  var hasEditGroupNamePermission = 0.obs;
  var hasAddMemberPermission = 0.obs;
  var uploading = false.obs;
  TextEditingController groupNameController = TextEditingController();
  @override
  void initState() {
    if (widget.groupId != null) {
      groupId.value = widget.groupId!;
      getGroup(
          groupId: groupId.value,
          onSuccess: (val) {

            GetGroupDetailModel groupDetailModel =
                getGroupDetailModelFromJson(val);
            print(groupDetailModel.groupPermission![0].permission!.exitGroup!);
            groupName.value = groupDetailModel.name!;
            groupNameController.text = groupName.value;
            groupImage.value = groupDetailModel.groupImage!;
            createdBy.value = groupDetailModel.createdBy!;
            participantList.value = groupDetailModel.users!;
            hasEditGroupNamePermission.value = groupDetailModel
                .groupPermission![0].permission!.changeGroupName!;
            hasAddMemberPermission.value =
                groupDetailModel.groupPermission![0].permission!.addMember!;
            hasEditGroupImagePermission.value = groupDetailModel
                .groupPermission![0].permission!.addProfilePicture!;
            hasRemoveMemberPermission.value =
                groupDetailModel.groupPermission![0].permission!.removeMember!;
            hasExitGRPPermission.value =
                groupDetailModel.groupPermission![0].permission!.exitGroup!;
            hasDeleteGRPPermission.value =
                groupDetailModel.groupPermission![0].permission!.deleteChat!;
            loadData.value = false;
          });
    }

    super.initState();
  }

  var imageURL = "".obs;
  var profileImage = File("").obs;
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      resizeToAvoidBottomInset: false,
      body: SafeArea(
        child: Container(
          color: const Color(0xffE5E5EA),
          child: Obx(
            () => loadData.value
                ? const Center(
                    child: CircularProgressIndicator(),
                  )
                : SingleChildScrollView(
                    child: Column(
                      children: [
                        appBar(),
                        Container(
                          width: MediaQuery.of(context).size.width,
                          color: Colors.white,
                          child: Column(
                            children: [
                              const SizedBox(
                                height: 24,
                              ),
                              InkWell(
                                onTap: editName.value
                                    ? () async {
                                        var pickedFile = File("");
                                        FilePickerResult? result =
                                            await FilePicker.platform.pickFiles(
                                          type: FileType.media,
                                          withData: true,
                                        );
                                        pickedFile =
                                            File(result!.paths[0].toString());
                                        profileImage.value = pickedFile;
                                        uploadImageApi(file: pickedFile.path);
                                      }
                                    : null,
                                child: Obx(
                                  () => Stack(
                                    alignment: AlignmentDirectional.center,
                                    children: [
                                      !profileImage.value.isAbsolute
                                          ? Common.profileNetworkImage(
                                              borderRadius: 200.0,
                                              height: 200.0,
                                              width: 200.0,
                                              placeHolder: ImageResource
                                                  .groupPlaceHolder,
                                              imageUrl: groupImage.value,
                                            )
                                          : ClipRRect(
                                              borderRadius:
                                                  BorderRadius.circular(200),
                                              child: Container(
                                                color: Colors.black12,
                                                child: Image.file(
                                                  File(
                                                    profileImage.value.path,
                                                  ),
                                                  height: 200.0,
                                                  width: 200.0,
                                                  fit: BoxFit.cover,
                                                ),
                                              ),
                                            ),
                                      uploading.value
                                          ? const Center(
                                        child: CircularProgressIndicator(),
                                      ): hasEditGroupImagePermission.value == 1
                                          ? const Icon(
                                              Icons.camera_alt_outlined,
                                              color: Colors.white,
                                              size: 30.0,
                                            )
                                          : const SizedBox.shrink(),

                                    ],
                                  ),
                                ),
                              ),
                              const SizedBox(height: 16),
                              Obx(
                                () => editName.value
                                    ? Padding(
                                        padding: const EdgeInsets.only(
                                            left: 16.0, right: 16.0),
                                        child: TextField(
                                          controller: groupNameController,
                                          decoration: InputDecoration(
                                            hintText: "Enter User Name",
                                            hintStyle: CustomTextStyles.medium(
                                                fontSize: 14.0,
                                                fontColor: Colors.black38),
                                            filled: true,
                                            fillColor: const Color(0xffEEEEEF),
                                            border: searchCommonBorderStyle,
                                            enabledBorder:
                                                searchCommonBorderStyle,
                                            disabledBorder:
                                                searchCommonBorderStyle,
                                            focusedBorder:
                                                searchCommonBorderStyle,
                                            focusedErrorBorder:
                                                searchCommonBorderStyle,
                                            errorBorder:
                                                searchCommonBorderStyle,
                                            contentPadding:
                                                const EdgeInsets.only(
                                                    left: 6, right: 6),
                                          ),
                                        ),
                                      )
                                    : Padding(
                                        padding: const EdgeInsets.only(
                                            top: 16, right: 16),
                                        child: Obx(
                                          ()=> Text(
                                            groupName.value,
                                            style: CustomTextStyles.medium(
                                                fontSize: 26.0,
                                                fontColor: Colors.black),
                                          ),
                                        ),
                                      ),
                              ),
                              const Padding(
                                padding:
                                    EdgeInsets.only(top: 10.0, bottom: 10.0),
                                child:
                                    Divider(color: Colors.black26, height: 1.0),
                              ),
                              Text(
                                "Group - ${participantList.length} participants",
                                style: CustomTextStyles.normal(
                                    fontSize: 20.0, fontColor: Colors.black),
                              ),
                              const SizedBox(height: 16),
                            ],
                          ),
                        ),
                        const SizedBox(height: 10),
                        Container(
                          color: Colors.white,
                          child: Column(
                            children: [
                              Padding(
                                padding: const EdgeInsets.only(
                                    left: 12.0,
                                    right: 12.0,
                                    top: 16.0,
                                    bottom: 16.0),
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: [
                                    Text(
                                      "${participantList.length} participants",
                                      style: CustomTextStyles.medium(
                                          fontSize: 16.0,
                                          fontColor: const Color(0xff667B88)),
                                    ),
                                    hasAddMemberPermission.value == 1
                                        ? InkWell(
                                            onTap: () {
                                              Navigator.push(
                                                context,
                                                MaterialPageRoute(
                                                    builder: (context) =>
                                                        ContactListView(
                                                          forMakeGroup: true,
                                                          forAddMember: true,
                                                          participentList:
                                                              participantList,
                                                          groupId:
                                                              groupId.value,
                                                          theme: widget.theme,
                                                        )),
                                              ).then((value) {
                                                if (value != null) {
                                                  var groupDetailParam = {
                                                    "groupId": groupId.value,
                                                    "userId":
                                                        AppStorages.myUserId,
                                                    "secretKey":
                                                        AppStorages.secKey
                                                  };
                                                  socket.emit('get-group',
                                                      groupDetailParam);
                                                }
                                              });
                                            },
                                            child: Text(
                                              "+ Add Member",
                                              style: CustomTextStyles.medium(
                                                  fontSize: 16.0,
                                                  fontColor:
                                                      const Color(0xff0F659E)),
                                            ),
                                          )
                                        : const SizedBox.shrink()
                                  ],
                                ),
                              ),
                              Obx(
                                () => ListView.separated(
                                    shrinkWrap: true,
                                    physics:
                                        const NeverScrollableScrollPhysics(),
                                    padding: const EdgeInsets.only(
                                      left: 12.0,
                                      right: 12.0,
                                      bottom: 14.0,
                                    ),
                                    itemBuilder: (context, index) {
                                      return Row(
                                        children: [
                                          Common.profileNetworkImage(
                                            borderRadius: 56.0,
                                            height: 56.0,
                                            width: 56.0,
                                            placeHolder: ImageResource
                                                .profilePlaceHolder,
                                            imageUrl: participantList[index]
                                                    .profilePicture ??
                                                "",
                                          ),
                                          const SizedBox(width: 10.0),
                                          Expanded(
                                            child: Text(
                                              participantList[index].userId! ==
                                                      AppStorages.myUserId
                                                  ? "${participantList[index].name!} (You)"
                                                  : participantList[index]
                                                      .name!,
                                              style: CustomTextStyles.medium(
                                                  fontSize: 14.0,
                                                  fontColor: Colors.black),
                                              overflow: TextOverflow.ellipsis,
                                            ),
                                          ),
                                          participantList[index].userId ==
                                                  createdBy.value
                                              ? Container(
                                                  decoration: BoxDecoration(
                                                    borderRadius:
                                                        BorderRadius.circular(
                                                            4),
                                                    color: themeColor,
                                                  ),
                                                  padding: const EdgeInsets
                                                          .symmetric(
                                                      vertical: 6,
                                                      horizontal: 6),
                                                  child: Text(
                                                    "Admin",
                                                    style:
                                                        CustomTextStyles.normal(
                                                      fontSize: 10.0,
                                                      fontColor: const Color(
                                                          0xff0F659E),
                                                    ),
                                                  ),
                                                )
                                              : participantList[index].userId ==
                                                      AppStorages.myUserId
                                                  ? const SizedBox.shrink()
                                                  : hasRemoveMemberPermission
                                                              .value ==
                                                          1
                                                      ? InkWell(
                                                          onTap: () {
                                                            removeMemberApi(
                                                                index,
                                                                userId:
                                                                    participantList[
                                                                            index]
                                                                        .userId);
                                                          },
                                                          child: Container(
                                                            decoration:
                                                                BoxDecoration(
                                                              borderRadius:
                                                                  BorderRadius
                                                                      .circular(
                                                                          4),
                                                              color: const Color(
                                                                  0xffFFE2E5),
                                                            ),
                                                            padding:
                                                                const EdgeInsets
                                                                        .symmetric(
                                                                    vertical: 6,
                                                                    horizontal:
                                                                        6),
                                                            child: Text(
                                                              "Remove",
                                                              style:
                                                                  CustomTextStyles
                                                                      .normal(
                                                                fontSize: 10.0,
                                                                fontColor:
                                                                    const Color(
                                                                        0xffF64E89),
                                                              ),
                                                            ),
                                                          ),
                                                        )
                                                      : const SizedBox.shrink()
                                        ],
                                      );
                                    },
                                    separatorBuilder: (context, index) {
                                      return const SizedBox(height: 16);
                                    },
                                    itemCount: participantList.length),
                              ),
                            ],
                          ),
                        ),
                         const SizedBox(height: 10),
                        hasExitGRPPermission.value == 1
                            ? InkWell(
                                onTap: () {
                                  exitGroup(
                                      userId: AppStorages.myUserId,
                                      groupId: groupId.value,
                                      context: context);
                                },
                                child: Container(
                                  color: Colors.white,
                                  padding: const EdgeInsets.all(20),
                                  width: MediaQuery.of(context).size.width,
                                  child: Row(
                                    children: [
                                      const Icon(
                                        Icons.login,
                                        color: Colors.red,
                                      ),
                                      const SizedBox(width: 8),
                                      Text(
                                        "Exit Group",
                                        style: CustomTextStyles.medium(
                                            fontSize: 16.0,
                                            fontColor: Colors.red),
                                      )
                                    ],
                                  ),
                                ),
                              )
                            : const SizedBox.shrink(),
                        const SizedBox(height: 10),
                        hasDeleteGRPPermission.value == 1
                            ? InkWell(
                                onTap: () {
                                  deleteGroup(
                                      userId: AppStorages.myUserId,
                                      groupId: groupId.value,
                                      context: context);
                                },
                                child: Container(
                                  color: Colors.white,
                                  padding: const EdgeInsets.all(20),
                                  width:  MediaQuery.of(context).size.width,
                                  child: Row(
                                    children: [
                                      const Icon(
                                        Icons.delete,
                                        color: Colors.red,
                                      ),
                                      const SizedBox(
                                        width: 8,
                                      ),
                                      Text(
                                        "Delete Group",
                                        style: CustomTextStyles.medium(
                                            fontSize: 16.0,
                                            fontColor: Colors.red),
                                      )
                                    ],
                                  ),
                                ),
                              )
                            : const SizedBox.shrink()
                      ],
                    ),
                  ),
          ),
        ),
      ),
    );
  }

  var searchCommonBorderStyle = const OutlineInputBorder(
    borderSide: BorderSide(
      color: Color(0xffE6E6E6),
    ),
  );
  Widget appBar() {
    return Row(
      children: [
        IconButton(
            onPressed: () {
              Navigator.pop(context);
            },
            icon: const Icon(
              Icons.clear,
              size: 30.0,
            )),
        const SizedBox(
          width: 8,
        ),
        Expanded(
          child: Text(
            "Group Info",
            style: CustomTextStyles.medium(
                fontSize: 18.0, fontColor: Colors.black),
          ),
        ),
        Obx(() => hasEditGroupNamePermission.value == 1
            ? Padding(
                padding: const EdgeInsets.only(right: 10),
                child: InkWell(
                  onTap: () {
                    if (editName.value) {

                      var param = {
                        "groupId": groupId.value,
                        "name": groupNameController.text,
                        "groupImage": imageURL.isNotEmpty?imageURL.value:groupImage.value,
                        "secretKey":AppStorages.secKey
                      };
                      updateGroupInfo(param, context: context);
                      editName.value = false;
                    } else {
                      editName.value = true;
                    }
                  },
                  child: Obx(
                    () => Text(
                      editName.value ? "Save" : "Edit",
                      style: CustomTextStyles.medium(
                        fontSize: 20.0,
                        fontColor: Colors.black,
                      ),
                    ),
                  ),
                ),
              )
            : const SizedBox.shrink())
      ],
    );
  }


  uploadImageApi({contentType, file, fileName, pickdFile, fileType}) async {

    uploading.value = true;
    var body = {
      "secretKey": AppStorages.secKey.toString(),
      "userId":AppStorages.myUserId.toString(),
      "groupId":"",
      "type":"image",
      "isChat":"0",
    };
    Webservice.uploadImage(
      uri: Global.uploadFileNew,
      body: body,
      hasBearer: true,
      onSuccess: (response) {

        var res = jsonDecode(response);
        imageURL.value = res['file'];
        uploading.value = false;
      },
      onFailure: (v) {
        uploading.value = false;
      }, file: File(file), fileType: 'selectFile',
    );

  }

  removeMemberApi(index, {userId}) {
    var body = {
      "userId": userId,
      "groupId": groupId.value,
      "secretKey": AppStorages.secKey,
    };
    Webservice.postRequest(
      uri: Global.removeMember,
      body: body,
      baseUrl: Global.baseApiUrl,
      hasBearer: false,
      onSuccess: (response) {
        var groupDetailParam = {
          "groupId": groupId.value,
          "userId": AppStorages.myUserId,
          "secretKey": AppStorages.secKey
        };
        socket.emit('get-group', groupDetailParam);
      },
      onFailure: (v) {
        // imageUploadLoader.value = false;
        // isImageLoading.value = false;
      },
    );
  }
}
